@@ -74,6 +74,7 @@ Stanza::Error::Error(int _type, int _condition, const QString &_text, const QDom
 	condition = _condition;
 	text = _text;
 	appSpec = _appSpec;
+	originalCode = 0;
 }
 
 
@@ -461,6 +462,7 @@ Stanza::Stanza()
 
 Stanza::Stanza(Stream *s, Kind k, const Jid &to, const QString &type, const QString &id)
 {
+	Q_ASSERT(s);
 	d = new Private;
 
 	Kind kind;
@@ -470,7 +472,8 @@ Stanza::Stanza(Stream *s, Kind k, const Jid &to, const QString &type, const QStr
 		kind = Message;
 
 	d->s = s;
-	d->e = d->s->doc().createElementNS(s->baseNS(), Private::kindToString(kind));
+	if(d->s)
+		d->e = d->s->doc().createElementNS(s->baseNS(), Private::kindToString(kind));
 	if(to.isValid())
 		setTo(to);
 	if(!type.isEmpty())
@@ -481,6 +484,7 @@ Stanza::Stanza(Stream *s, Kind k, const Jid &to, const QString &type, const QStr
 
 Stanza::Stanza(Stream *s, const QDomElement &e)
 {
+	Q_ASSERT(s);
 	d = 0;
 	if(e.namespaceURI() != s->baseNS())
 		return;
