@@ -26,71 +26,9 @@
 //Added by qt3to4:
 #include <QList>
 
+#include "xmpp_xmlcommon.h"
 #define NS_XML     "http://www.w3.org/XML/1998/namespace"
 
-static QDomElement textTag(QDomDocument *doc, const QString &name, const QString &content)
-{
-	QDomElement tag = doc->createElement(name);
-	QDomText text = doc->createTextNode(content);
-	tag.appendChild(text);
-
-	return tag;
-}
-
-static QString tagContent(const QDomElement &e)
-{
-	// look for some tag content
-	for(QDomNode n = e.firstChild(); !n.isNull(); n = n.nextSibling()) {
-		QDomText i = n.toText();
-		if(i.isNull())
-			continue;
-		return i.data();
-	}
-
-	return "";
-}
-
-static QDateTime stamp2TS(const QString &ts)
-{
-	if(ts.length() != 17)
-		return QDateTime();
-
-	int year  = ts.mid(0,4).toInt();
-	int month = ts.mid(4,2).toInt();
-	int day   = ts.mid(6,2).toInt();
-
-	int hour  = ts.mid(9,2).toInt();
-	int min   = ts.mid(12,2).toInt();
-	int sec   = ts.mid(15,2).toInt();
-
-	QDate xd;
-	xd.setYMD(year, month, day);
-	if(!xd.isValid())
-		return QDateTime();
-
-	QTime xt;
-	xt.setHMS(hour, min, sec);
-	if(!xt.isValid())
-		return QDateTime();
-
-	return QDateTime(xd, xt);
-}
-
-static QString TS2stamp(const QDateTime &d)
-{
-	QString str;
-
-	str.sprintf("%04d%02d%02dT%02d:%02d:%02d",
-		d.date().year(),
-		d.date().month(),
-		d.date().day(),
-		d.time().hour(),
-		d.time().minute(),
-		d.time().second());
-
-	return str;
-}
-
 namespace XMPP
 {
 
@@ -902,11 +840,11 @@ HTMLElement::HTMLElement()
 {
 }
 
-HTMLElement::HTMLElement(const QDomElement &body) : body_(body) {}
+HTMLElement::HTMLElement(const QDomElement &body) { setBody(body); }
 
 void HTMLElement::setBody(const QDomElement &body)
 {
-	body_ = body;
+	body_ = doc_.importNode(body, true).toElement();
 }
 
 const QDomElement& HTMLElement::body() const
@@ -970,17 +908,17 @@ public:
 	QString eventId;
 	QString xencrypted, invite;
 	ChatState chatState;
+	MessageReceipt messageReceipt;
 	QString nick;
 	HttpAuthRequest httpAuthRequest;
 	XData xdata;
 	QMap<QString,HTMLElement> htmlElements;
- 	QDomElement wb;
+ 	QDomElement sxe;
 	
 	int mucStatus;
 	QList<MUCInvite> mucInvites;
 	MUCDecline mucDecline;
 	QString mucPassword;
-	int mucHistoryMaxChars, mucHistoryMaxStanzas, mucHistorySeconds;
 
 	bool spooled, wasEncrypted;
 };
@@ -1003,6 +941,7 @@ Message::Message(const Jid &to)
 	d->errorCode = -1;*/
 	d->chatState = StateNone;
 	d->mucStatus = -1;
+	d->messageReceipt = ReceiptNone;
 }
 
 //! \brief Constructs a copy of Message object
@@ -1332,6 +1271,16 @@ void Message::setChatState(ChatState state)
 	d->chatState = state;
 }
 
+MessageReceipt Message::messageReceipt() const
+{
+	return d->messageReceipt;
+}
+
+void Message::setMessageReceipt(MessageReceipt messageReceipt)
+{
+	d->messageReceipt = messageReceipt;
+}
+
 QString Message::xencrypted() const
 {
 	return d->xencrypted;
@@ -1427,14 +1376,14 @@ const XData& Message::getForm() const
 	return d->xdata;
 }
 
-const QDomElement& Message::whiteboard() const
+const QDomElement& Message::sxe() const
 {
-	return d->wb;
+	return d->sxe;
 }
 
-void Message::setWhiteboard(const QDomElement& e)
+void Message::setSxe(const QDomElement& e)
 {
-	d->wb = e;
+	d->sxe = e;
 }
 
 bool Message::spooled() const
@@ -1569,6 +1518,21 @@ Stanza Message::toStanza(Stream *stream) const
 		}
 	}
 
+	// message receipt
+	QString messageReceiptNS = "urn:xmpp:receipts";
+	if (d->messageReceipt != ReceiptNone) {
+		switch(d->messageReceipt) {
+			case ReceiptRequest:
+				s.appendChild(s.createElement(messageReceiptNS, "request"));
+				break;
+			case ReceiptReceived:
+				s.appendChild(s.createElement(messageReceiptNS, "received"));
+				break;
+			default: 
+				break;
+		}
+	}
+
 	// xencrypted
 	if(!d->xencrypted.isEmpty())
 		s.appendChild(s.createTextElement("jabber:x:encrypted", "x", d->xencrypted));
@@ -1603,9 +1567,9 @@ Stanza Message::toStanza(Stream *stream) const
 		s.appendChild(s.createTextElement("http://jabber.org/protocol/nick", "nick", d->nick));
 	}
 
-	// wb
-	if(!d->wb.isNull()) {
-		s.appendChild(d->wb);
+	// sxe
+	if(!d->sxe.isNull()) {
+		s.appendChild(d->sxe);
 	}
 
 	// muc
@@ -1657,7 +1621,7 @@ bool Message::fromStanza(const Stanza &s, int timeZoneOffset)
 
 	QDomElement root = s.element();
 
-	QDomNodeList nl = root.childNodes();
+	XDomNodeList nl = root.childNodes();
 	int n;
 	for(n = 0; n < nl.count(); ++n) {
 		QDomNode i = nl.item(n);
@@ -1707,7 +1671,7 @@ bool Message::fromStanza(const Stanza &s, int timeZoneOffset)
 		d->error = s.error();
 
 	// xhtml-im
-	nl = root.elementsByTagNameNS("http://jabber.org/protocol/xhtml-im", "html");
+	nl = childElementsByTagNameNS(root, "http://jabber.org/protocol/xhtml-im", "html");
 	if (nl.count()) {
 		nl = nl.item(0).childNodes();
 		for(n = 0; n < nl.count(); ++n) {
@@ -1720,7 +1684,7 @@ bool Message::fromStanza(const Stanza &s, int timeZoneOffset)
 	}
 
 	// timestamp
-	QDomElement t = root.elementsByTagNameNS("jabber:x:delay", "x").item(0).toElement();
+	QDomElement t = childElementsByTagNameNS(root, "jabber:x:delay", "x").item(0).toElement();
 	if(!t.isNull()) {
 		d->timeStamp = stamp2TS(t.attribute("stamp"));
 		d->timeStamp = d->timeStamp.addSecs(timeZoneOffset * 3600);
@@ -1733,7 +1697,7 @@ bool Message::fromStanza(const Stanza &s, int timeZoneOffset)
 
 	// urls
 	d->urlList.clear();
-	nl = root.elementsByTagNameNS("jabber:x:oob", "x");
+	nl = childElementsByTagNameNS(root, "jabber:x:oob", "x");
 	for(n = 0; n < nl.count(); ++n) {
 		QDomElement t = nl.item(n).toElement();
 		Url u;
@@ -1744,7 +1708,7 @@ bool Message::fromStanza(const Stanza &s, int timeZoneOffset)
 	
     // events
 	d->eventList.clear();
-	nl = root.elementsByTagNameNS("jabber:x:event", "x");
+	nl = childElementsByTagNameNS(root, "jabber:x:event", "x");
 	if (nl.count()) {
 		nl = nl.item(0).childNodes();
 		for(n = 0; n < nl.count(); ++n) {
@@ -1765,25 +1729,33 @@ bool Message::fromStanza(const Stanza &s, int timeZoneOffset)
 
 	// Chat states
 	QString chatStateNS = "http://jabber.org/protocol/chatstates";
-	t = root.elementsByTagNameNS(chatStateNS, "active").item(0).toElement();
+	t = childElementsByTagNameNS(root, chatStateNS, "active").item(0).toElement();
 	if(!t.isNull())
 		d->chatState = StateActive;
-	t = root.elementsByTagNameNS(chatStateNS, "composing").item(0).toElement();
+	t = childElementsByTagNameNS(root, chatStateNS, "composing").item(0).toElement();
 	if(!t.isNull())
 		d->chatState = StateComposing;
-	t = root.elementsByTagNameNS(chatStateNS, "paused").item(0).toElement();
+	t = childElementsByTagNameNS(root, chatStateNS, "paused").item(0).toElement();
 	if(!t.isNull())
 		d->chatState = StatePaused;
-	t = root.elementsByTagNameNS(chatStateNS, "inactive").item(0).toElement();
+	t = childElementsByTagNameNS(root, chatStateNS, "inactive").item(0).toElement();
 	if(!t.isNull())
 		d->chatState = StateInactive;
-	t = root.elementsByTagNameNS(chatStateNS, "gone").item(0).toElement();
+	t = childElementsByTagNameNS(root, chatStateNS, "gone").item(0).toElement();
 	if(!t.isNull())
 		d->chatState = StateGone;
-	
+
+	// message receipts
+	QString messageReceiptNS = "urn:xmpp:receipts";
+	t = childElementsByTagNameNS(root, messageReceiptNS, "request").item(0).toElement();
+	if(!t.isNull())
+		d->messageReceipt = ReceiptRequest;
+	t = childElementsByTagNameNS(root, messageReceiptNS, "received").item(0).toElement();
+	if(!t.isNull())
+		d->messageReceipt = ReceiptReceived;
 
 	// xencrypted
-	t = root.elementsByTagNameNS("jabber:x:encrypted", "x").item(0).toElement();
+	t = childElementsByTagNameNS(root, "jabber:x:encrypted", "x").item(0).toElement();
 	if(!t.isNull())
 		d->xencrypted = t.text();
 	else
@@ -1791,7 +1763,7 @@ bool Message::fromStanza(const Stanza &s, int timeZoneOffset)
 		
 	// addresses
 	d->addressList.clear();
-	nl = root.elementsByTagNameNS("http://jabber.org/protocol/address", "addresses");
+	nl = childElementsByTagNameNS(root, "http://jabber.org/protocol/address", "addresses");
 	if (nl.count()) {
 		QDomElement t = nl.item(0).toElement();
 		nl = t.elementsByTagName("address");
@@ -1802,7 +1774,7 @@ bool Message::fromStanza(const Stanza &s, int timeZoneOffset)
 	
 	// roster item exchange
 	d->rosterExchangeItems.clear();
-	nl = root.elementsByTagNameNS("http://jabber.org/protocol/rosterx", "x");
+	nl = childElementsByTagNameNS(root, "http://jabber.org/protocol/rosterx", "x");
 	if (nl.count()) {
 		QDomElement t = nl.item(0).toElement();
 		nl = t.elementsByTagName("item");
@@ -1814,27 +1786,27 @@ bool Message::fromStanza(const Stanza &s, int timeZoneOffset)
 	}
 
 	// invite
-	t = root.elementsByTagNameNS("jabber:x:conference", "x").item(0).toElement();
+	t = childElementsByTagNameNS(root, "jabber:x:conference", "x").item(0).toElement();
 	if(!t.isNull())
 		d->invite = t.attribute("jid");
 	else
 		d->invite = QString();
 	
 	// nick
-	t = root.elementsByTagNameNS("http://jabber.org/protocol/nick", "nick").item(0).toElement();
+	t = childElementsByTagNameNS(root, "http://jabber.org/protocol/nick", "nick").item(0).toElement();
 	if(!t.isNull())
 		d->nick = t.text();
 	else
 		d->nick = QString();
 
-	// wb
-	t = root.elementsByTagNameNS("http://jabber.org/protocol/svgwb", "wb").item(0).toElement();
+	// sxe
+	t = childElementsByTagNameNS(root, "http://jabber.org/protocol/sxe", "sxe").item(0).toElement();
 	if(!t.isNull())
-		d->wb = t;
+		d->sxe = t;
 	else
-		d->wb = QDomElement();
+		d->sxe = QDomElement();
 
-	t = root.elementsByTagNameNS("http://jabber.org/protocol/muc#user", "x").item(0).toElement();
+	t = childElementsByTagNameNS(root, "http://jabber.org/protocol/muc#user", "x").item(0).toElement();
 	if(!t.isNull()) {
 		for(QDomNode muc_n = t.firstChild(); !muc_n.isNull(); muc_n = muc_n.nextSibling()) {
 			QDomElement muc_e = muc_n.toElement();
@@ -1858,7 +1830,7 @@ bool Message::fromStanza(const Stanza &s, int timeZoneOffset)
 	}
 
 	// http auth
-	t = root.elementsByTagNameNS("http://jabber.org/protocol/http-auth", "confirm").item(0).toElement();
+	t = childElementsByTagNameNS(root, "http://jabber.org/protocol/http-auth", "confirm").item(0).toElement();
 	if(!t.isNull()){
 		d->httpAuthRequest = HttpAuthRequest(t);
 	}
@@ -1867,7 +1839,7 @@ bool Message::fromStanza(const Stanza &s, int timeZoneOffset)
 	}
 
 	// data form
-	t = root.elementsByTagNameNS("jabber:x:data", "x").item(0).toElement();
+	t = childElementsByTagNameNS(root, "jabber:x:data", "x").item(0).toElement();
 	if(!t.isNull()){
 		d->xdata.fromXml(t);
 	}
@@ -2258,10 +2230,7 @@ bool Status::isAvailable() const
 
 bool Status::isAway() const
 {
-	if(v_show == "away" || v_show == "xa" || v_show == "dnd")
-		return true;
-
-	return false;
+	return (v_show == "away" || v_show == "xa" || v_show == "dnd");
 }
 
 bool Status::isInvisible() const
