@@ -57,7 +57,9 @@
 #include "resourcemenu.h"
 #include "shortcutmanager.h"
 #include "xmpp_message.h"
-
+#include "textutil.h"
+#include "bookmarkmanagedlg.h"
+#include "bookmarkmanager.h"
 
 static inline int rankStatus(int status) 
 {
@@ -146,10 +148,11 @@ ContactProfile::ContactProfile(PsiAccount *pa, const QString &name, ContactView
 
 	d->self = 0;
 
-	if(!unique)
+	if (!unique) {
 		d->cvi = new ContactViewItem(name, this, d->cv);
-	else
+	} else {
 		d->cvi = 0;
+	}
 
 	d->oldstate = -2;
 
@@ -214,8 +217,9 @@ const QString & ContactProfile::name() const
 void ContactProfile::setName(const QString &name)
 {
 	d->name = name;
-	if(d->cvi)
+	if (d->cvi) {
 		d->cvi->setProfileName(name);
+	}
 }
 
 void ContactProfile::setName(const char *s)
@@ -603,6 +607,7 @@ void ContactProfile::resetAllContactItemNames()
 		Q3PtrListIterator<ContactViewItem> cvi_it(e->cvi);
 		for(ContactViewItem *i; (i = cvi_it.current()); ++cvi_it) {
 			i->resetName();
+			contactView()->filterContact(i);
 		}
 	}
 }
@@ -639,7 +644,7 @@ void ContactProfile::setAlert(const Jid &j, const PsiIcon *anim)
 		for(ContactViewItem *i; (i = it.current()); ++it)
 			i->setAlert(anim);
 
-		if(option.scrollTo)
+		if(PsiOptions::instance()->getOption("options.ui.contactlist.ensure-contact-visible-on-event").toBool())
 			ensureVisible(e);
 	}
 }
@@ -809,10 +814,10 @@ void ContactProfile::updateGroupInfo(ContactViewItem *group)
 		else {
 			total = group->childCount();
 		}
-		if (option.showGroupCounts)
+		if (PsiOptions::instance()->getOption("options.ui.contactlist.show-group-counts").toBool())
 			group->setGroupInfo(QString("(%1/%2)").arg(online).arg(total));
 	}
-	else if (option.showGroupCounts) {
+	else if (PsiOptions::instance()->getOption("options.ui.contactlist.show-group-counts").toBool()) {
 		int inGroup = contactSizeFromCVGroup(group);
 		group->setGroupInfo(QString("(%1)").arg(inGroup));
 	}
@@ -863,14 +868,17 @@ void ContactProfile::updateGroups()
 			if(e->u.isAvailable())
 				++totalOnline;
 		}
-		if(d->cvi && option.showGroupCounts)
+		if(d->cvi && PsiOptions::instance()->getOption("options.ui.contactlist.show-group-counts").toBool())
 			d->cvi->setGroupInfo(QString("(%1/%2)").arg(totalOnline).arg(d->roster.count()));
 	}
 
 	{
 		Q3PtrListIterator<ContactViewItem> it(d->groups);
 		for(ContactViewItem *g; (g = it.current()); ++it)
+		{
 			updateGroupInfo(g);
+			contactView()->filterGroup(g);
+		}
 	}
 }
 
@@ -916,7 +924,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 		am->insertItem(/*IconsetFactory::iconPixmap("psi/edit/clear"),*/ tr("Update MOTD"), 3);
 		am->insertItem(IconsetFactory::icon("psi/remove").icon(), tr("Delete MOTD"), 4);
 
-		const int status_start = 16;
+		const int status_start = 15;
 		Q3PopupMenu *sm = new Q3PopupMenu(&pm);
 		sm->insertItem(PsiIconset::instance()->status(STATUS_ONLINE).icon(),	status2txt(STATUS_ONLINE),	STATUS_ONLINE		+ status_start);
 		if (PsiOptions::instance()->getOption("options.ui.menu.status.chat").toBool())
@@ -943,6 +951,22 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 		pm.insertItem(tr("Avatar"), avatarm, 14);
 		pm.setItemEnabled(14, d->pa->serverInfoManager()->hasPEP());
 #endif
+		const int bookmarks_start = STATUS_CHAT + status_start + 1; // STATUS_CHAT is the highest value of the states
+		QMenu *bookmarks = new QMenu(&pm);
+		bookmarks->insertItem(tr("Manage..."), bookmarks_start);
+		if (d->pa->bookmarkManager()->isAvailable()) {
+			int idx = 1;
+			bookmarks->insertSeparator();
+			foreach(ConferenceBookmark c, psiAccount()->bookmarkManager()->conferences()) {
+				bookmarks->insertItem(QString(tr("Join %1")).arg(c.name()), bookmarks_start + idx);
+				idx++;
+			}
+		}
+		else {
+			bookmarks->setEnabled(false);
+		}
+
+		pm.insertItem(tr("Bookmarks"), bookmarks);
 
 		pm.insertSeparator();
 		pm.insertItem(IconsetFactory::icon("psi/addContact").icon(), tr("&Add a contact"), 7);
@@ -953,7 +977,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 		pm.insertItem(IconsetFactory::icon("psi/xml").icon(), tr("&XML Console"), 10);
 		pm.insertSeparator();
 		pm.insertItem(IconsetFactory::icon("psi/account").icon(), tr("&Modify Account..."), 0);
-
+		
 		if (PsiOptions::instance()->getOption("options.ui.menu.account.admin").toBool()) {
 			pm.insertSeparator();
 			pm.setItemEnabled(pm.insertItem(tr("&Admin"), am), online);
@@ -1011,10 +1035,23 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 		else if(x == 13  && pm.isItemEnabled(14)) {
 			emit actionUnsetAvatar();
 		}
-		else if(x >= status_start) {
+		else if(x >= status_start && x <= STATUS_CHAT + status_start) { // STATUS_CHAT is the highest value of the states
 			int status = x - status_start;
 			d->pa->changeStatus(status);
 		}
+		else if(x == bookmarks_start) {
+			BookmarkManageDlg *dlg = d->pa->findDialog<BookmarkManageDlg*>();
+			if(dlg) {
+				bringToFront(dlg);
+			} else {
+				dlg = new BookmarkManageDlg(d->pa);
+				dlg->show();
+			}
+		}
+		else if (x > bookmarks_start) {
+			ConferenceBookmark c = psiAccount()->bookmarkManager()->conferences()[x - bookmarks_start - 1];
+			psiAccount()->actionJoin(c, true);
+		}
 	}
 	else if(i->type() == ContactViewItem::Group) {
 		QString gname = i->groupName();
@@ -1022,7 +1059,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 
 		if (PsiOptions::instance()->getOption("options.ui.message.enabled").toBool())
 			pm.insertItem(IconsetFactory::icon("psi/sendMessage").icon(), tr("Send message to group"), 0);
-		if(!option.lockdown.roster) {
+		if(!PsiOptions::instance()->getOption("options.ui.contactlist.lockdown-roster").toBool()) {
 			// disable if it's not a user group
 			if(!online || i->groupType() != ContactViewItem::gUser || gname == ContactView::tr("Hidden")) {
 				d->cv->qa_ren->setEnabled(false);
@@ -1046,7 +1083,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 		int x = pm.exec(pos);
 
 		// restore actions
-		if(option.lockdown.roster)
+		if(PsiOptions::instance()->getOption("options.ui.contactlist.lockdown-roster").toBool())
 			d->cv->qa_ren->setEnabled(false);
 		else
 			d->cv->qa_ren->setEnabled(true);
@@ -1117,7 +1154,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 
 		Q3PopupMenu pm;
 
-		if(!self && !inList && !isPrivate && !option.lockdown.roster) {
+		if(!self && !inList && !isPrivate && !PsiOptions::instance()->getOption("options.ui.contactlist.lockdown-roster").toBool()) {
 			pm.insertItem(IconsetFactory::icon("psi/addContact").icon(), tr("Add/Authorize to contact list"), 10);
 			if(!online)
 				pm.setItemEnabled(10, false);
@@ -1261,14 +1298,14 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 
 		if(!self) {
 			if(inList) {
-				if(!option.lockdown.roster) {
+				if(!PsiOptions::instance()->getOption("options.ui.contactlist.lockdown-roster").toBool()) {
 					d->cv->qa_ren->setEnabled(online);
 					d->cv->qa_ren->addTo(&pm);
 				}
 			}
 
 			if(!isAgent) {
-				if(inList && !option.lockdown.roster) {
+				if(inList && !PsiOptions::instance()->getOption("options.ui.contactlist.lockdown-roster").toBool()) {
 					Q3PopupMenu *gm = new Q3PopupMenu(&pm);
 
 					gm->setCheckable(true);
@@ -1323,7 +1360,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 			}
 		}
 
-		if(inList && !option.lockdown.roster) {
+		if(inList && !PsiOptions::instance()->getOption("options.ui.contactlist.lockdown-roster").toBool()) {
 			Q3PopupMenu *authm = new Q3PopupMenu (&pm);
 
 			authm->insertItem(tr("Resend authorization to"), 6);
@@ -1336,7 +1373,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 		}
 
 		if(!self) {
-			if(!option.lockdown.roster) {
+			if(!PsiOptions::instance()->getOption("options.ui.contactlist.lockdown-roster").toBool()) {
 				if(online || !inList)
 					d->cv->qa_rem->setEnabled(true);
 				else
@@ -1377,7 +1414,8 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 		int x = pm.exec(pos);
 
 		// restore actions
-		if(option.lockdown.roster) {
+		d->cv->qa_logon->setEnabled(true);
+		if(PsiOptions::instance()->getOption("options.ui.contactlist.lockdown-roster").toBool()) {
 			d->cv->qa_ren->setEnabled(false);
 			d->cv->qa_rem->setEnabled(false);
 		}
@@ -1823,72 +1861,15 @@ public:
 		cv = _cv;
 		autoRosterResizeInProgress = false;
 
-		// type ahead
-		typeAheadTimer = new QTimer(cv);
-		connect(typeAheadTimer, SIGNAL(timeout()), SLOT(resetTypeAhead()));
 	}
 
 	ContactView *cv;
 	QTimer *animTimer, *recalculateSizeTimer;
 	Q3PtrList<ContactProfile> profiles;
-	QString typeAhead;
-	QTimer *typeAheadTimer;
 	QSize lastSize;
 	bool autoRosterResizeInProgress;
 
-	bool doTypeAhead(QKeyEvent *e)
-	{
-		typeAheadTimer->start(3000, TRUE); // reset type ahead in 3 seconds of inactivity
 
-		QString text = e->text().lower();
-		if ( text.isEmpty() ) {
-			// Shift key could be used for entering '@', '#' and other extended chars
-			// and other modifier keys could be used for layout changing
-
-			//if ( e->key() != Qt::Key_Shift )
-			//	typeAhead = QString::null;
-			return false;
-		}
-
-		bool searchFromStart = true;
-		if ( typeAhead.isEmpty() )
-			searchFromStart = false;
-
-		bool found = false;
-		typeAhead += text;
-
-		Q3ListViewItemIterator it(cv);
-		ContactViewItem *item;
-
-		if ( !searchFromStart && cv->currentItem() ) {
-			for ( ; (item = (ContactViewItem *)it.current()); ++it) {
-				if ( item == cv->currentItem() ) {
-					++it; // we want the search to start from next contact
-					break;
-				}
-			}
-		}
-
-		for ( ; (item = (ContactViewItem *)it.current()); ++it) {
-			//qWarning("->%s | %s | %s", text.latin1(), item->text(0).lower().latin1(), typeAhead.latin1());
-			if ( item->text(0).lower().startsWith(typeAhead) ) {
-				found = true;
-				cv->setSelected(item, true);
-				cv->ensureItemVisible(item);
-				item->optionsUpdate();
-				item->repaint();
-				break;
-			}
-		}
-
-		//qWarning("---------------\n");
-		if ( !found ) {
-			typeAhead = text;
-			return false;
-		}
-
-		return true;
-	}
 
 public slots:
 	/*
@@ -1920,7 +1901,7 @@ public slots:
 				autoRosterResizeInProgress = true;
 
 				QRect desktop = qApp->desktop()->availableGeometry( (QWidget *)topParent );
-				if ( option.autoRosterSizeGrowTop ) {
+				if ( PsiOptions::instance()->getOption("options.ui.contactlist.grow-roster-upwards").toBool() ) {
 					int newy = topParent->y() - dh;
 
 					// check, if we need to move roster lower
@@ -1961,8 +1942,8 @@ public slots:
 		int top_offs    = abs( desktop.top()    - topParent->frameGeometry().top() );
 		int bottom_offs = abs( desktop.bottom() - topParent->frameGeometry().bottom() );
 
-		option.autoRosterSizeGrowTop = bottom_offs < top_offs;
-		//qWarning("growTop = %d", option.autoRosterSizeGrowTop);
+		PsiOptions::instance()->setOption("options.ui.contactlist.grow-roster-upwards", (bool) (bottom_offs < top_offs));
+		//qWarning("growTop = %d", PsiOptions::instance()->getOption("options.ui.contactlist.automatically-resize-roster").toBool()GrowTop);
 	}
 	
 	/*
@@ -1992,10 +1973,6 @@ private slots:
 		autoRosterResizeInProgress = false;
 	}
 
-	void resetTypeAhead()
-	{
-		typeAhead = QString::null;
-	}
 };
 
 ContactView::ContactView(QWidget *parent, const char *name)
@@ -2027,13 +2004,14 @@ ContactView::ContactView(QWidget *parent, const char *name)
 	connect(this, SIGNAL(doubleClicked(Q3ListViewItem *)),SLOT(qlv_doubleclick(Q3ListViewItem *)) );
 	connect(this, SIGNAL(contextMenuRequested(Q3ListViewItem *, const QPoint &, int)), SLOT(qlv_contextMenuRequested(Q3ListViewItem *, const QPoint &, int)));
 
-	v_showOffline = true;
-	v_showAway = true;
-	v_showHidden = true;
-	v_showAgents = true;
-	v_showSelf = true;
+	v_showOffline = PsiOptions::instance()->getOption("options.ui.contactlist.show.offline-contacts").toBool();
+	v_showAway = PsiOptions::instance()->getOption("options.ui.contactlist.show.away-contacts").toBool();
+	v_showHidden = PsiOptions::instance()->getOption("options.ui.contactlist.show.hidden-contacts-group").toBool();
+	v_showAgents = PsiOptions::instance()->getOption("options.ui.contactlist.show.agent-contacts").toBool();
+	v_showSelf = PsiOptions::instance()->getOption("options.ui.contactlist.show.self-contact").toBool();
 	v_showStatusMsg = PsiOptions::instance()->getOption("options.ui.contactlist.status-messages.show").toBool();
 
+
 	d->lastSize = QSize( 0, 0 );
 
 	// animation timer
@@ -2069,7 +2047,7 @@ ContactView::ContactView(QWidget *parent, const char *name)
 	qa_vcard = new IconAction("", "psi/vCard", tr("User &Info"), 0, this);
 	connect(qa_vcard, SIGNAL(activated()), SLOT(doVCard()));
 
-	if(option.lockdown.roster) {
+	if(PsiOptions::instance()->getOption("options.ui.contactlist.lockdown-roster").toBool()) {
 		qa_ren->setEnabled(false);
 		qa_rem->setEnabled(false);
 	}
@@ -2077,6 +2055,8 @@ ContactView::ContactView(QWidget *parent, const char *name)
 	optionsUpdate();
 	setAcceptDrops(true);
 	viewport()->setAcceptDrops(true);
+	
+	filterString_ = QString();
 }
 
 ContactView::~ContactView()
@@ -2085,6 +2065,96 @@ ContactView::~ContactView()
 	delete d;
 }
 
+bool ContactView::filterContact(ContactViewItem *item, bool refineSearch)
+{
+	if (!item) {
+		return false;
+	}
+	if (item->type() != ContactViewItem::Contact) {
+		return false;
+	}
+	if (filterString_.isNull()) {
+		return true;	
+	}
+	//if refineSearch, only search still visible items
+	if (refineSearch && !item->isVisible()) {
+		return false;		
+	}
+	if (TextUtil::rich2plain(item->text(0)).contains(filterString_,Qt::CaseInsensitive))
+	{
+		ensureItemVisible(item);
+		item->setVisible(true);
+		item->optionsUpdate();			
+	}
+	else
+	{
+		item->setVisible(false);
+	}
+	item->repaint();	
+	return item->isVisible();
+}
+
+bool ContactView::filterGroup(ContactViewItem *group, bool refineSearch)
+{
+	if (!group) {
+		return false;	
+	} else if (group->type() != ContactViewItem::Group) {
+		return false;
+	} else if (filterString_.isNull()) {
+		return true;
+	}
+	//if refine_search, only search still visible items
+	if (refineSearch && !group->isVisible()) {
+		return false;	
+	}
+	group->setVisible(true); //if not refined search
+	
+	//iterate over children
+	Q3ListViewItemIterator it(group);
+	bool groupContainsAFinding = false;
+	ContactViewItem *item = static_cast<ContactViewItem*>(group->firstChild());
+	while(item) {
+		if (filterContact(item,refineSearch))
+			groupContainsAFinding = true;
+        item = static_cast<ContactViewItem*>(item->nextSibling());
+	}
+	if (groupContainsAFinding == false) {
+		group->setVisible(false);		
+	}
+	group->repaint();
+	return groupContainsAFinding;
+}
+
+void ContactView::setFilter(QString const &text)
+{
+	bool refineSearch = text.startsWith(filterString_);
+	filterString_ = text;
+	
+	Q3ListViewItemIterator it(d->cv);
+	for (ContactViewItem *item; (item = (ContactViewItem *)it.current()); ++it)
+	{	
+		if (item->type() == ContactViewItem::Group) {
+			filterGroup(item, refineSearch);
+		}
+	}
+}
+
+void ContactView::clearFilter()
+{
+	filterString_=QString();
+	Q3ListViewItemIterator it(d->cv);
+	for (ContactViewItem *item; (item = (ContactViewItem *)it.current()); ++it) 
+	{
+		if (item->type() != ContactViewItem::Contact && item->type() != ContactViewItem::Group) {
+			continue;
+		}
+		item->setVisible(true);
+		item->optionsUpdate();
+		item->repaint();
+	}	
+}
+
+
 QTimer *ContactView::animTimer() const
 {
 	return d->animTimer;
@@ -2110,21 +2180,39 @@ void ContactView::unlink(ContactProfile *cp)
 void ContactView::keyPressEvent(QKeyEvent *e)
 {
 	int key = e->key();
-	if(key == Qt::Key_Enter || key == Qt::Key_Return)
+	if(key == Qt::Key_Enter || key == Qt::Key_Return) {
 		doEnter();
-	else if(key == Qt::Key_Space && d->typeAhead.isEmpty())
+	} else if(key == Qt::Key_Space /*&& d->typeAhead.isEmpty()*/) {
 		doContext();
-	else if (key == Qt::Key_Home   || key == Qt::Key_End      ||
+	} else if (key == Qt::Key_Home   || key == Qt::Key_End      ||
 		 key == Qt::Key_PageUp || key == Qt::Key_PageDown ||
 		 key == Qt::Key_Up     || key == Qt::Key_Down     ||
 		 key == Qt::Key_Left   || key == Qt::Key_Right) {
 
-		d->typeAhead = QString::null;
+		//d->typeAhead = QString::null;
 		Q3ListView::keyPressEvent(e);
-	}
-	else {
-		if (!d->doTypeAhead(e))
+#ifdef Q_WS_MAC
+	} else if (e->modifiers() == Qt::ControlModifier) {
+		Q3ListView::keyPressEvent(e);
+#endif 
+	} else {
+		QString text = e->text().lower();
+		if (text.isEmpty()) {
 			Q3ListView::keyPressEvent(e);
+		}
+		else {
+			bool printable = true;
+			foreach (QChar ch, text) {
+				if (!ch.isPrint()) {
+					Q3ListView::keyPressEvent(e);
+					printable = false;
+					break;
+				}
+			}
+			if (printable) {
+				emit searchInput(text);
+			}
+		}
 	}
 }
 
@@ -2132,6 +2220,8 @@ void ContactView::setShowOffline(bool x)
 {
 	bool oldstate = v_showOffline;
 	v_showOffline = x;
+	PsiOptions::instance()->setOption("options.ui.contactlist.show.offline-contacts", x);
+
 
 	if(v_showOffline != oldstate) {
 		showOffline(v_showOffline);
@@ -2150,7 +2240,8 @@ void ContactView::setShowAway(bool x)
 {
 	bool oldstate = v_showAway;
 	v_showAway = x;
-
+	PsiOptions::instance()->setOption("options.ui.contactlist.show.away-contacts", x);
+		
 	if(v_showAway != oldstate) {
 		showAway(v_showAway);
 
@@ -2168,6 +2259,7 @@ void ContactView::setShowHidden(bool x)
 {
 	bool oldstate = v_showHidden;
 	v_showHidden = x;
+	PsiOptions::instance()->setOption("options.ui.contactlist.show.hidden-contacts-group", x);
 
 	if(v_showHidden != oldstate) {
 		showHidden(v_showHidden);
@@ -2207,6 +2299,7 @@ void ContactView::setShowSelf(bool x)
 {
 	if (v_showSelf != x) {
 		v_showSelf = x;
+		PsiOptions::instance()->setOption("options.ui.contactlist.show.self-contact", x);
 		showSelf(v_showSelf);
 
 		Q3PtrListIterator<ContactProfile> it(d->profiles);
@@ -2245,6 +2338,7 @@ void ContactView::setShowAgents(bool x)
 {
 	bool oldstate = v_showAgents;
 	v_showAgents = x;
+	PsiOptions::instance()->setOption("options.ui.contactlist.show.agent-contacts", x);
 
 	if(v_showAgents != oldstate) {
 		showAgents(v_showAgents);
@@ -2262,7 +2356,7 @@ void ContactView::setShowAgents(bool x)
 // right-click context menu
 void ContactView::qlv_contextMenuRequested(Q3ListViewItem *lvi, const QPoint &pos, int c)
 {
-	if(option.useleft)
+	if(PsiOptions::instance()->getOption("options.ui.contactlist.use-left-click").toBool())
 		return;
 
 	qlv_contextPopup(lvi, pos, c);
@@ -2296,9 +2390,9 @@ void ContactView::qlv_singleclick(int button, Q3ListViewItem *i, const QPoint &p
 		if (button == Qt::LeftButton && item->type() == ContactViewItem::Group && pix && viewport()->mapFromGlobal(pos).x() <= pix->width() + treeStepSize()) {
 			setOpen(item, !item->isOpen());
 		}
-		else if(option.useleft) {
+		else if(PsiOptions::instance()->getOption("options.ui.contactlist.use-left-click").toBool()) {
 			if(button == Qt::LeftButton) {
-				if(option.singleclick) {
+				if(PsiOptions::instance()->getOption("options.ui.contactlist.use-single-click").toBool()) {
 					qlv_contextPopup(i, pos, c);
 				}
 				else {
@@ -2308,7 +2402,7 @@ void ContactView::qlv_singleclick(int button, Q3ListViewItem *i, const QPoint &p
 					QTimer::singleShot(QApplication::doubleClickInterval()/2, this, SLOT(leftClickTimeOut()));
 				}
 			}
-			else if(option.singleclick && button == Qt::RightButton) {
+			else if(PsiOptions::instance()->getOption("options.ui.contactlist.use-single-click").toBool() && button == Qt::RightButton) {
 				if(item->type() == ContactViewItem::Contact)
 					item->contactProfile()->scActionDefault(item);
 			}
@@ -2317,14 +2411,14 @@ void ContactView::qlv_singleclick(int button, Q3ListViewItem *i, const QPoint &p
 			//if(button == QListView::RightButton) {
 			//	qlv_contextPopup(i, pos, c);
 			//}
-			/*else*/if(button == Qt::LeftButton && option.singleclick) {
+			/*else*/if(button == Qt::LeftButton && PsiOptions::instance()->getOption("options.ui.contactlist.use-single-click").toBool()) {
 				if(item->type() == ContactViewItem::Contact)
 					item->contactProfile()->scActionDefault(item);
 			}
 		}
 	}
 
-	d->typeAhead = "";
+	//d->typeAhead = "";
 }
 
 void ContactView::qlv_doubleclick(Q3ListViewItem *i)
@@ -2334,13 +2428,13 @@ void ContactView::qlv_doubleclick(Q3ListViewItem *i)
 	if(!i)
 		return;
 
-	if(option.singleclick)
+	if(PsiOptions::instance()->getOption("options.ui.contactlist.use-single-click").toBool())
 		return;
 
 	ContactViewItem *item = (ContactViewItem *)i;
 	item->contactProfile()->scActionDefault(item);
 
-	d->typeAhead = "";
+	//d->typeAhead = "";
 }
 
 void ContactView::qlv_itemRenamed(Q3ListViewItem *lvi, int, const QString &text)
@@ -2361,13 +2455,13 @@ void ContactView::optionsUpdate()
 {
 	// set the font
 	QFont f;
-	f.fromString(option.font[fRoster]);
+	f.fromString(PsiOptions::instance()->getOption("options.ui.look.font.contactlist").toString());
 	Q3ListView::setFont(f);
 
 	// set the text and background colors
 	QPalette mypal = Q3ListView::palette();
-	mypal.setColor(QColorGroup::Text, option.color[cOnline]);
-	mypal.setColor(QColorGroup::Base, option.color[cListBack]);
+	mypal.setColor(QColorGroup::Text, PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.status.online").value<QColor>());
+	mypal.setColor(QColorGroup::Base, PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.background").value<QColor>());
 	Q3ListView::setPalette(mypal);
 
 	// reload the icons
@@ -2382,7 +2476,7 @@ void ContactView::optionsUpdate()
 	setShortcuts();
 
 	// resize if necessary
-	if (option.autoRosterSize)
+	if (PsiOptions::instance()->getOption("options.ui.contactlist.automatically-resize-roster").toBool())
 		recalculateSize();
 
 	update();
@@ -2538,7 +2632,7 @@ Q3DragObject *ContactView::dragObject()
 
 bool ContactView::allowResize() const
 {
-	if ( !option.autoRosterSize )
+	if ( !PsiOptions::instance()->getOption("options.ui.contactlist.automatically-resize-roster").toBool() )
 		return false;
 
 	if ( window()->isMaximized() )
@@ -3032,15 +3126,15 @@ void ContactViewItem::paintCell(QPainter *p, const QColorGroup & cg, int column,
 		QColorGroup xcg = cg;
 
 		if(d->status == STATUS_AWAY || d->status == STATUS_XA)
-			xcg.setColor(QColorGroup::Text, option.color[cAway]);
+			xcg.setColor(QColorGroup::Text, PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.status.away").value<QColor>());
 		else if(d->status == STATUS_DND)
-			xcg.setColor(QColorGroup::Text, option.color[cDND]);
+			xcg.setColor(QColorGroup::Text, PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.status.do-not-disturb").value<QColor>());
 		else if(d->status == STATUS_OFFLINE)
-			xcg.setColor(QColorGroup::Text, option.color[cOffline]);
+			xcg.setColor(QColorGroup::Text, PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.status.offline").value<QColor>());
 
 		if(d->animatingNick) {
-			xcg.setColor(QColorGroup::Text, d->animateNickColor ? option.color[cAnimFront] : option.color[cAnimBack]);
-			xcg.setColor(QColorGroup::HighlightedText, d->animateNickColor ? option.color[cAnimFront] : option.color[cAnimBack]);
+			xcg.setColor(QColorGroup::Text, d->animateNickColor ? PsiOptions::instance()->getOption("options.ui.look.contactlist.status-change-animation.color1").value<QColor>() : PsiOptions::instance()->getOption("options.ui.look.contactlist.status-change-animation.color2").value<QColor>());
+			xcg.setColor(QColorGroup::HighlightedText, d->animateNickColor ? PsiOptions::instance()->getOption("options.ui.look.contactlist.status-change-animation.color1").value<QColor>() : PsiOptions::instance()->getOption("options.ui.look.contactlist.status-change-animation.color2").value<QColor>());
 		}
 
 		RichListViewItem::paintCell(p, xcg, column, width, alignment);
@@ -3070,24 +3164,24 @@ void ContactViewItem::paintCell(QPainter *p, const QColorGroup & cg, int column,
 		QColorGroup xcg = cg;
 
 		if(type_ == Profile) {
- 			xcg.setColor(QColorGroup::Text, option.color[cProfileFore]);
+ 			xcg.setColor(QColorGroup::Text, PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.profile.header-foreground").value<QColor>());
 			#if QT_VERSION < 0x040301
-				xcg.setColor(QColorGroup::Background, option.color[cProfileBack]);
+				xcg.setColor(QColorGroup::Background, PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.profile.header-background").value<QColor>());
 			#else
- 				xcg.setColor(QColorGroup::Base, option.color[cProfileBack]);
+ 				xcg.setColor(QColorGroup::Base, PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.profile.header-background").value<QColor>());
  			#endif
 			
 		}
 		else if(type_ == Group) {
 			QFont f = p->font();
-			f.setPointSize(option.smallFontSize);
+			f.setPointSize(common_smallFontSize);
 			p->setFont(f);
-			xcg.setColor(QColorGroup::Text, option.color[cGroupFore]);
-			if (!option.clNewHeadings) {
+			xcg.setColor(QColorGroup::Text, PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.grouping.header-foreground").value<QColor>());
+			if (!PsiOptions::instance()->getOption("options.ui.look.contactlist.use-slim-group-headings").toBool()) {
  				#if QT_VERSION < 0x040301
-					xcg.setColor(QColorGroup::Background, option.color[cGroupBack]);
+					xcg.setColor(QColorGroup::Background, PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.grouping.header-background").value<QColor>());
 				#else
- 					xcg.setColor(QColorGroup::Base, option.color[cGroupBack]);
+ 					xcg.setColor(QColorGroup::Base, PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.grouping.header-background").value<QColor>());
 				#endif
 			}
 		} 
@@ -3111,7 +3205,7 @@ void ContactViewItem::paintCell(QPainter *p, const QColorGroup & cg, int column,
 			p->setPen(xcg.text());
 
 		QFont f_info = p->font();
-		f_info.setPointSize(option.smallFontSize);
+		f_info.setPointSize(common_smallFontSize);
 		p->setFont(f_info);
 		QFontMetrics fm_info(p->font());
 		//int info_x = width - fm_info.width(d->groupInfo) - 8;
@@ -3119,23 +3213,23 @@ void ContactViewItem::paintCell(QPainter *p, const QColorGroup & cg, int column,
 		int info_y = ((height() - fm_info.height()) / 2) + fm_info.ascent();
 		p->drawText((info_x > x ? info_x : x), info_y, d->groupInfo);
 
-		if(type_ == Group && option.clNewHeadings && !isSelected()) {
+		if(type_ == Group && PsiOptions::instance()->getOption("options.ui.look.contactlist.use-slim-group-headings").toBool() && !isSelected()) {
 			x += fm.width(d->groupInfo) + 8;
 			if(x < width - 8) {
 				int h = (height() / 2) - 1;
-				p->setPen(QPen(option.color[cGroupBack]));
+				p->setPen(QPen(PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.grouping.header-background").value<QColor>()));
 				p->drawLine(x, h, width - 8, h);
 				h++;
-				p->setPen(QPen(option.color[cGroupFore]));
+				p->setPen(QPen(PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.grouping.header-foreground").value<QColor>()));
 				/*int h = height() / 2;
 
-				p->setPen(QPen(option.color[cGroupBack], 2));*/
+				p->setPen(QPen(PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.grouping.header-background").value<QColor>(), 2));*/
 				p->drawLine(x, h, width - 8, h);
 			}
 		} 
 		else {
-			if (option.outlineHeadings) {
-				p->setPen(QPen(option.color[cGroupFore]));
+			if (PsiOptions::instance()->getOption("options.ui.look.contactlist.use-outlined-group-headings").toBool()) {
+				p->setPen(QPen(PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.grouping.header-foreground").value<QColor>()));
 				p->drawRect(0, 0, width, height());
 			}
 		}
@@ -3205,7 +3299,7 @@ int ContactViewItem::compare(Q3ListViewItem *lvi, int, bool) const
 		if(i->type() == Group)
 			ret = -1;
 		else {
-			if ( option.rosterContactSortStyle == Options::ContactSortStyle_Status ) {
+			if ( PsiOptions::instance()->getOption("options.ui.contactlist.contact-sort-style").toString() == "status" ) {
 				ret = rankStatus(d->status) - rankStatus(i->status());
 				if(ret == 0)
 					ret = text(0).lower().localeAwareCompare(i->text(0).lower());
@@ -3220,7 +3314,7 @@ int ContactViewItem::compare(Q3ListViewItem *lvi, int, bool) const
 		if(i->type() == Contact)
 			ret = 1;
 		else if(i->type() == Group) {
-			if ( option.rosterGroupSortStyle == Options::GroupSortStyle_Rank ) {
+			if ( PsiOptions::instance()->getOption("options.ui.contactlist.group-sort-style").toString() == "rank") {
 				int ourRank   = d->groupData().rank;
 				int theirRank = i->d->groupData().rank;
 
@@ -3233,7 +3327,7 @@ int ContactViewItem::compare(Q3ListViewItem *lvi, int, bool) const
 			}
 		}
 		else if(i->type() == Profile) {
-			if ( option.rosterAccountSortStyle == Options::AccountSortStyle_Rank ) {
+			if ( PsiOptions::instance()->getOption("options.ui.contactlist.account-sort-style").toString() == "rank" ) {
 				int ourRank = d->groupData().rank;
 				int theirRank = i->d->groupData().rank;
 
@@ -3322,7 +3416,7 @@ void ContactViewItem::resetName(bool forceNoStatusMsg)
 				statusMsg.replace(">","&gt;");
 				statusMsg = statusMsg.simplifyWhiteSpace();
 				if (!statusMsg.isEmpty())
-					s += "<br><font size=-1 color='" + option.color[cStatus].name() + "'><nobr>" + statusMsg + "</nobr></font>";
+					s += "<br><font size=-1 color='" + PsiOptions::instance()->getOption("options.ui.look.colors.contactlist.status-messages").value<QColor>().name() + "'><nobr>" + statusMsg + "</nobr></font>";
 			}
 			else {
 				statusMsg.replace('\n'," ");
