@@ -450,7 +450,7 @@ void AttachView::addUrlList(const UrlList &list)
 AddUrlDlg::AddUrlDlg(QWidget *parent)
 :QDialog(parent)
 {
-	if ( option.brushedMetal )
+	if ( PsiOptions::instance()->getOption("options.ui.mac.use-brushed-metal-windows").toBool() )
 		setAttribute(Qt::WA_MacMetalStyle);
 	setupUi(this);
 #ifndef Q_WS_MAC
@@ -557,18 +557,11 @@ public slots:
 		if ( !dlg->isActiveWindow() )
 		     return;
 
-		QString text;
+		QString text = icon->defaultText();
 
-		QHash<QString,QString> itext = icon->text();
-		for (QHash<QString,QString>::ConstIterator it = itext.begin() ; it != itext.end(); ++it) {
-			if ( !it->isEmpty() ) {
-				text = (*it) + " ";
-				break;
-			}
+		if ( !text.isEmpty() ) {
+			mle->insert( text + " " );
 		}
-
-		if ( !text.isEmpty() )
-			mle->insert( text );
 	}
 
 	void addEmoticon(QString text) {
@@ -587,7 +580,7 @@ EventDlg::EventDlg(const QString &to, PsiCon *psi, PsiAccount *pa)
 	: AdvancedWidget<QWidget>(0)
 {
 	setAttribute(Qt::WA_DeleteOnClose);
-  	if ( option.brushedMetal )
+  	if ( PsiOptions::instance()->getOption("options.ui.mac.use-brushed-metal-windows").toBool() )
 		setAttribute(Qt::WA_MacMetalStyle);	
 	d = new Private(this);
 	d->composing = true;
@@ -612,7 +605,7 @@ EventDlg::EventDlg(const QString &to, PsiCon *psi, PsiAccount *pa)
 	d->le_to->setText(expandAddresses(to, false));
 	d->le_to->setCursorPosition(0);
 
-	if(option.grabUrls) {
+	if(PsiOptions::instance()->getOption("options.ui.message.auto-grab-urls-from-clipboard").toBool()) {
 		// url in clipboard?
 		QClipboard *cb = QApplication::clipboard();
 		QString text = cb->text(QClipboard::Clipboard);
@@ -820,7 +813,7 @@ void EventDlg::init()
 	QHBoxLayout *hb3 = new QHBoxLayout(vb1);
 
 //	if(d->composing /* && config->showsubject */) {
-	if(option.showSubjects) {
+	if(PsiOptions::instance()->getOption("options.ui.message.show-subjects").toBool()) {
 		// third row
 		l = new QLabel(tr("Subject:"), this);
 		hb3->addWidget(l);
@@ -996,9 +989,13 @@ void EventDlg::init()
 
 	updatePGP();
 	connect(d->pa, SIGNAL(pgpKeyChanged()), SLOT(updatePGP()));
-	connect(d->pa, SIGNAL(encryptedMessageSent(int, bool, int)), SLOT(encryptedMessageSent(int, bool, int)));
+	connect(d->pa, SIGNAL(encryptedMessageSent(int, bool, int, const QString &)), SLOT(encryptedMessageSent(int, bool, int, const QString &)));
 
-	resize(option.sizeEventDlg);
+	if (PsiOptions::instance()->getOption("options.ui.message.size").toSize().isValid()) {
+		resize(PsiOptions::instance()->getOption("options.ui.message.size").toSize());
+	} else {
+		resize(defaultSize());
+	}
 	optionsUpdate();
 
 	//ShortcutManager::connect("common.close", this, SLOT(close()));
@@ -1206,7 +1203,7 @@ void EventDlg::to_changeResource(const QString &r)
 
 void EventDlg::to_tryComplete()
 {
-	if(!option.jidComplete)
+	if(!PsiOptions::instance()->getOption("options.ui.message.use-jid-auto-completion").toBool())
 		return;
 
 	QString str = d->le_to->text();
@@ -1343,18 +1340,18 @@ void EventDlg::optionsUpdate()
 {
 	// update the font
 	QFont f;
-	f.fromString(option.font[fMessage]);
+	f.fromString(PsiOptions::instance()->getOption("options.ui.look.font.message").toString());
 	d->mle->setFont(f);
 
 	// update status icon
 	doWhois(true);
 
-	if ( option.showCounter && d->composing )
+	if ( PsiOptions::instance()->getOption("options.ui.message.show-character-count").toBool() && d->composing )
 		d->lb_count->show();
 	else
 		d->lb_count->hide();
 
-	if ( option.useEmoticons )
+	if ( PsiOptions::instance()->getOption("options.ui.emoticons.use-emoticons").toBool() )
 		d->tb_icon->show();
 	else
 		d->tb_icon->hide();
@@ -1405,12 +1402,13 @@ void EventDlg::showEvent(QShowEvent *e)
 
 void EventDlg::resizeEvent(QResizeEvent *e)
 {
-	if(option.keepSizes)
-		option.sizeEventDlg = e->size();
+	if(PsiOptions::instance()->getOption("options.ui.remember-window-sizes").toBool())
+		PsiOptions::instance()->getOption("options.ui.message.size").toSize() = e->size();
 }
 
 void EventDlg::keyPressEvent(QKeyEvent *e)
 {
+	// FIXMEKEY
 	QKeySequence key = e->key() + ( e->modifiers() & ~Qt::KeypadModifier);
 	if(ShortcutManager::instance()->shortcuts("common.close").contains(key))
 		close();
@@ -1864,11 +1862,11 @@ void EventDlg::updateEvent(PsiEvent *e)
 		QString txt = xhtml ? m.html().toString("div") : TextUtil::plain2rich(m.body());
 
 		// show subject line if the incoming message has one
-		if(m.subject() != "" && !option.showSubjects)
+		if(m.subject() != "" && !PsiOptions::instance()->getOption("options.ui.message.show-subjects").toBool())
 			txt = "<p><font color=\"red\"><b>" + tr("Subject:") + " " + TextUtil::plain2rich(m.subject()) + "</b></font></p>" + (xhtml? "" : "<br>") + txt;
 
 		if (!xhtml) {
-			if(option.useEmoticons)
+			if(PsiOptions::instance()->getOption("options.ui.emoticons.use-emoticons").toBool())
 				txt = TextUtil::emoticonify(txt);
 			if( PsiOptions::instance()->getOption("options.ui.chat.legacy-formatting").toBool() )
 				txt = TextUtil::legacyFormat(txt);
@@ -2105,7 +2103,7 @@ void EventDlg::trySendEncryptedNext()
 	}
 }
 
-void EventDlg::encryptedMessageSent(int x, bool b, int e)
+void EventDlg::encryptedMessageSent(int x, bool b, int e, const QString &dtext)
 {
 	if(d->transid == -1)
 		return;
@@ -2130,8 +2128,9 @@ void EventDlg::encryptedMessageSent(int x, bool b, int e)
 			return;
 		}
 	}
-	else
-		QMessageBox::critical(this, tr("Error"), tr("There was an error trying to send the message encrypted.\nReason: %1.").arg(PGPUtil::instance().messageErrorString((QCA::SecureMessage::Error) e)));
+	else {
+		PGPUtil::showDiagnosticText(static_cast<QCA::SecureMessage::Error>(e), dtext);
+	}
 
 	d->le_to->setEnabled(true);
 	d->mle->setEnabled(true);
