@@ -26,19 +26,19 @@
 #include "psicon.h"
 #include "psiaccount.h"
 #include "userlist.h"
-#include "common.h"
 #include "iconwidget.h"
 #include "s5b.h"
 #include "busywidget.h"
 #include "filetransfer.h"
 #include "accountscombobox.h"
-#include "profiles.h"
 #include "psiiconset.h"
 #include "msgmle.h"
 #include "jidutil.h"
 #include "psitooltip.h"
 #include "psicontactlist.h"
 #include "accountlabel.h"
+#include "psioptions.h"
+#include "fileutil.h"
 
 typedef Q_UINT64 LARGE_TYPE;
 
@@ -446,6 +446,23 @@ void FileTransferHandler::ft_error(int x)
 
 void FileTransferHandler::trySend()
 {
+	// Since trySend comes from singleShot which is an "uncancelable"
+	//   action, we should protect that d->ft is valid, for good measure
+	if(!d->ft)
+		return;
+
+	// When FileTransfer emits error, you are not allowed to call
+	//   dataSizeNeeded() afterwards.  Simetime ago, we changed to using
+	//   QueuedConnection for error() signal delivery (see mapSignals()).
+	//   This made it possible to call dataSizeNeeded by accident between
+	//   the error() signal emit and the ft_error() slot invocation.  To
+	//   work around this problem, we'll check to see if the FileTransfer
+	//   is internally active by checking if s5bConnection() is null.
+	//   FIXME: this probably breaks other file transfer methods, whenever
+	//   we get those.  Probably we need a real fix in Iris..
+	if(!d->ft->s5bConnection())
+		return;
+
 	int blockSize = d->ft->dataSizeNeeded();
 	QByteArray a(blockSize, 0);
 	int r = 0;
@@ -585,8 +602,8 @@ FileRequestDlg::FileRequestDlg(const Jid &jid, PsiCon *psi, PsiAccount *pa, cons
 			QTimer::singleShot(0, this, SLOT(reject()));
 			return;
 		}
-		
-		option.lastPath = fi.dirPath();
+
+		FileUtil::setLastUsedSavePath(fi.dirPath());
 		le_fname->setText(QDir::convertSeparators(fi.filePath()));
 		lb_size->setText(tr("%1 byte(s)").arg(fi.size())); // TODO: large file support
 	}
@@ -753,21 +770,13 @@ void FileRequestDlg::unblockWidgets()
 
 void FileRequestDlg::chooseFile()
 {
-	while(1) {
-		if(option.lastPath.isEmpty())
-			option.lastPath = QDir::homeDirPath();
-		QString str = QFileDialog::getOpenFileName(this, tr("Choose a file"), option.lastPath, tr("All files (*)"));
-		if(!str.isEmpty()) {
-			QFileInfo fi(str);
-			if(!fi.exists()) {
-				QMessageBox::information(this, tr("Error"), tr("The file specified does not exist."));
-				continue;
-			}
-			option.lastPath = fi.dirPath();
-			le_fname->setText(QDir::convertSeparators(fi.filePath()));
-			lb_size->setText(tr("%1 byte(s)").arg(fi.size())); // TODO: large file support
-		}
-		break;
+	QString str = FileUtil::getOpenFileName(this,
+	                                        tr("Choose a file"),
+	                                        tr("All files (*)"));
+	if (!str.isEmpty()) {
+		QFileInfo fi(str);
+		le_fname->setText(QDir::convertSeparators(fi.filePath()));
+		lb_size->setText(tr("%1 byte(s)").arg(fi.size())); // TODO: large file support
 	}
 }
 
@@ -808,13 +817,13 @@ void FileRequestDlg::doStart()
 	}
 	else {
 		QString fname, savename;
-		if(option.lastSavePath.isEmpty())
-			option.lastSavePath = QDir::homeDirPath();
-		fname = QFileDialog::getSaveFileName(this, tr("Save As"), QDir(option.lastSavePath).filePath(d->fileName), tr("All files (*)"));
+		fname = FileUtil::getSaveFileName(this,
+		                                  tr("Save As"),
+		                                  d->fileName,
+		                                  tr("All files (*)"));
 		if(fname.isEmpty())
 			return;
 		QFileInfo fi(fname);
-		option.lastSavePath = fi.dirPath();
 		savename = fname + ".part";
 		fname = fi.fileName();
 
@@ -1533,7 +1542,7 @@ public:
 				}
 			}
 
-			pa->playSound(option.onevent[eFTComplete]);
+			pa->playSound(PsiOptions::instance()->getOption("options.ui.notifications.sounds.completed-file-transfer").toString());
 		}
 	}
 };
