@@ -24,6 +24,10 @@
 #include <QWidget>
 #include <QDialog>
 
+
+#include "advwidget.h"
+#include "tabbablewidget.h"
+
 #include "ui_groupchatdlg.h"
 #include "mucmanager.h"
 #include "advwidget.h"
@@ -56,14 +60,13 @@ protected:
 	void keyPressEvent(QKeyEvent *);
 };*/
 
-class GCMainDlg : public AdvancedWidget<QWidget>
+class GCMainDlg : public TabbableWidget
 {
 	Q_OBJECT
 public:
-	GCMainDlg(PsiAccount *, const Jid &);
+	GCMainDlg(PsiAccount *, const Jid &, TabManager *tabManager);
 	~GCMainDlg();
 
-	Jid jid() const;
 	PsiAccount* account() const;
 
 	void error(int, const QString &);
@@ -73,25 +76,32 @@ public:
 	void setPassword(const QString&);
 	const QString& nick() const;
 
+	// reimplemented
+	virtual TabbableWidget::State state() const;
+	virtual int unreadMessageCount() const;
+	virtual QString desiredCaption() const;
+
 protected:
 	void setShortcuts();
 
 	// reimplemented
-	void keyPressEvent(QKeyEvent *);
 	void dragEnterEvent(QDragEnterEvent *);
 	void dropEvent(QDropEvent *);
 	void closeEvent(QCloseEvent *);
 	void resizeEvent(QResizeEvent*);
-
-	void windowActivationChange(bool);
 	void mucInfoDialog(const QString& title, const QString& message, const Jid& actor, const QString& reason);
 
 signals:
 	void aSend(const Message &);
 
 public slots:
-	void optionsUpdate();
+	// reimplemented
+	virtual void deactivated();
+	virtual void activated();
+	virtual void ensureTabbedCorrectly();
 
+	void optionsUpdate();
+	
 private slots:
 	void scrollUp();
 	void scrollDown();
@@ -100,7 +110,6 @@ private slots:
 	void openFind();
 	void configureRoom();
 	void doFind(const QString &);
-	void flashAnimate();
 	void pa_updatedActivity();
 	void goDisc();
 	void goConn();
@@ -125,11 +134,10 @@ private:
 	Private *d;
 	Ui::GroupChatDlg ui_;
 
-	void doFlash(bool);
 	void doAlert();
-	void updateCaption();
 	void appendSysMsg(const QString &, bool, const QDateTime &ts=QDateTime());
 	void appendMessage(const Message &, bool);
+	void updateLastMsgTime(QDateTime t);
 	void setLooks();
 
 	void contextMenuEvent(QContextMenuEvent *);
@@ -138,6 +146,7 @@ private:
 	QMap<QString,int> nicks;
 	int nicknumber;
 	PsiOptions* options_;
+	QDateTime lastMsgTime_;
 };
 
 class GCFindDlg : public QDialog
