@@ -21,7 +21,7 @@
 #ifndef MAINWIN_H
 #define MAINWIN_H
 
-#include <Q3MainWindow>
+#include <QMainWindow>
 #include <QMap>
 #include <QList>
 #include <QString>
@@ -45,8 +45,7 @@ namespace XMPP {
 	class Status;
 }
 
-class MainWin : public AdvancedWidget<Q3MainWindow>
-//class MainWin : public Q3MainWindow
+class MainWin : public AdvancedWidget<QMainWindow>
 {
 	Q_OBJECT
 public:
@@ -56,18 +55,18 @@ public:
 	void setWindowOpts(bool onTop, bool asTool);
 	void setUseDock(bool);
 
+	void buildToolbars();
+
 	// evil stuff! remove ASAP!!
 	QStringList actionList;
 	QMap<QString, QAction*> actions;
 
 	ContactView *cvlist;
-	QList<PsiToolBar*> toolbars;
 
-	void buildToolbars();
-	void saveToolbarsPositions();
 	PsiCon *psiCon() const;
 
 protected:
+	// reimplemented
 	void closeEvent(QCloseEvent *);
 	void keyPressEvent(QKeyEvent *);
 	QMenuBar* mainMenuBar() const;
@@ -88,8 +87,6 @@ signals:
 	void accountInfo();
 	void recvNextEvent();
 
-	void geomChanged(QRect saveableGeometry);
-
 private slots:
 	void buildStatusMenu();
 	void buildOptionsMenu();
@@ -126,9 +123,15 @@ private slots:
 	void actPlaySoundsActivated (bool);
 	void actPublishTuneActivated (bool);
 	void actTipActivated();
+	void actDiagQCAPluginActivated();
+	void actDiagQCAKeyStoreActivated();
 
 	bool showDockMenu(const QPoint &);
 	void dockActivated();
+	
+	void searchClearClicked();
+	void searchTextEntered(QString const &text);
+	void searchTextStarted(QString const &text);
 
 	void registerAction( IconAction * );
 
@@ -153,10 +156,14 @@ private:
 	void updateCaption();
 	void updateTray();
 
+	void saveToolbarsState();
+	void loadToolbarsState();
+
 private:
 	class Private;
 	Private *d;
 	friend class Private;
+	QList<PsiToolBar*> toolbars_;
 };
 
 #endif
