@@ -69,11 +69,12 @@ public:
 	EventNotifierAction(QObject *parent, const char *name = 0);
 	~EventNotifierAction();
 
-	void setText(const QString &);
+	void setMessage(const QString &);
 	bool addTo (QWidget *w);
 
 	void hide();
 	void show();
+	void updateVisibility();
 
 	virtual IconAction *copy() const;
 	virtual EventNotifierAction &operator=( const EventNotifierAction & );
@@ -130,7 +131,7 @@ signals:
 	void doubleClicked();
 };
 
-class MAction : public IconAction
+class MAction : public IconActionGroup
 {
 	Q_OBJECT
 
@@ -138,7 +139,8 @@ public:
 	MAction(PsiIcon, const QString &, int id, PsiCon *, QObject *parent);
 	MAction(const QString &, int id, PsiCon *, QObject *parent);
 
-	bool addTo(QWidget *w);
+	// reimplemented
+	virtual bool addTo(QWidget *);
 
 	virtual IconAction *copy() const;
 	virtual MAction &operator=( const MAction & );
@@ -148,16 +150,19 @@ signals:
 
 private slots:
 	void numAccountsChanged();
-	void itemActivated(int n);
+	void actionActivated();
+	void slotActivated();
 
 protected:
-	void addingToolButton(IconToolButton *);
+	// reimplemented
+	virtual void doSetMenu(QMenu* menu);
 
 private:
-	class Private;
-	Private *d;
+	int id_;
+	PsiCon* controller_;
 
-	void init (PsiIcon, int id, PsiCon *psi);
+	void init(const QString& name, PsiIcon, int id, PsiCon* psi);
+	QList<PsiAccount*> accounts() const;
 };
 
 #endif
