@@ -6,9 +6,9 @@
 #include "psitoolbar.h"
 #include "iconaction.h"
 #include "psiactionlist.h"
+#include "psioptions.h"
 
 #include "ui_opt_lookfeel_toolbars.h"
-#include "ui_ui_positiontoolbar.h"
 
 #include <qlayout.h>
 #include <qpushbutton.h>
@@ -27,117 +27,11 @@
 class LookFeelToolbarsUI : public QWidget, public Ui::LookFeelToolbars
 {
 public:
-	LookFeelToolbarsUI() : QWidget() { setupUi(this); }
+	LookFeelToolbarsUI() : QWidget() {
+		setupUi(this);
+	}
 };
 
-//----------------------------------------------------------------------------
-// PositionOptionsTabToolbars
-//----------------------------------------------------------------------------
-
-class PositionOptionsTabToolbars : public QDialog, public Ui::PositionToolbarUI
-{
-	Q_OBJECT
-public:
-	PositionOptionsTabToolbars(QWidget *parent, Options::ToolbarPrefs *, int);
-	~PositionOptionsTabToolbars();
-
-	int n();
-	bool dirty;
-
-	bool eventFilter(QObject *watched, QEvent *e);
-
-signals:
-	void applyPressed();
-
-private slots:
-	void dataChanged();
-	void apply();
-
-private:
-	int id;
-	Options::ToolbarPrefs *tb;
-};
-
-PositionOptionsTabToolbars::PositionOptionsTabToolbars(QWidget *parent, Options::ToolbarPrefs *_tb, int _id)
-: QDialog(parent)
-{
-	setupUi(this);
-	setModal(true);
-	tb = _tb;
-	id = _id;
-
-
-	connect(pb_ok, SIGNAL(clicked()), SLOT(apply()));
-	connect(pb_ok, SIGNAL(clicked()), SLOT(accept()));
-	connect(pb_apply, SIGNAL(clicked()), SLOT(apply()));
-	connect(pb_cancel, SIGNAL(clicked()), SLOT(reject()));
-
-	connect(cb_dock, SIGNAL(highlighted(int)), SLOT(dataChanged()));
-	sb_index->installEventFilter( this );
-	sb_extraOffset->installEventFilter( this );
-	connect(ck_nl, SIGNAL(toggled(bool)), SLOT(dataChanged()));
-
-	le_name->setText( tb->name );
-	if ( tb->dock >= Qt::DockUnmanaged && tb->dock <= Qt::DockTornOff ) {
-		cb_dock->setCurrentItem( tb->dock + Qt::DockMinimized - Qt::DockTornOff );
-	}
-	else {
-		cb_dock->setCurrentItem( tb->dock - Qt::DockTop );
-	}
-	sb_index->setValue( tb->index );
-	sb_extraOffset->setValue( tb->extraOffset );
-	ck_nl->setChecked( tb->nl );
-
-	dirty = false;
-	pb_apply->setEnabled(false);
-
-	resize(sizeHint());
-}
-
-PositionOptionsTabToolbars::~PositionOptionsTabToolbars()
-{
-}
-
-int PositionOptionsTabToolbars::n()
-{
-	return id;
-}
-
-void PositionOptionsTabToolbars::dataChanged()
-{
-	dirty = true;
-	pb_apply->setEnabled(true);
-}
-
-void PositionOptionsTabToolbars::apply()
-{
-	tb->dirty = true;
-	if ( cb_dock->currentItem() >= 0 && cb_dock->currentItem() < 5 ) {
-		// Top, Bottom, Left, Right and Minimised
-		tb->dock = (Qt::ToolBarDock)(cb_dock->currentItem() + Qt::DockTop);
-	}
-	else {
-		// Unmanaged and TornOff
-		tb->dock = (Qt::ToolBarDock)(cb_dock->currentItem() - (Qt::DockMinimized - Qt::DockTornOff));
-	}
-
-	tb->index = sb_index->value();
-	tb->extraOffset = sb_extraOffset->value();
-	tb->nl = ck_nl->isChecked();
-
-	if ( dirty )
-		emit applyPressed();
-	dirty = false;
-	pb_apply->setEnabled(false);
-}
-
-bool PositionOptionsTabToolbars::eventFilter(QObject *watched, QEvent *e)
-{
-	if ( watched->inherits("QSpinBox") && e->type() == QEvent::KeyRelease )
-		dataChanged();
-	return false;
-}
-
 //----------------------------------------------------------------------------
 // OptionsTabToolbars
 //----------------------------------------------------------------------------
@@ -145,54 +39,37 @@ bool PositionOptionsTabToolbars::eventFilter(QObject *watched, QEvent *e)
 class OptionsTabToolbars::Private
 {
 public:
-	struct ToolbarItem {
-		QString group;
-		int index;
-		ToolbarItem() {
-			group = "";
-			index = -1;
-		}
-		ToolbarItem( QString _w, int _i ) {
-			group = _w;
-			index = _i;
-		}
-	};
+	QMap<QString, ToolbarPrefs> toolbars;
 
-	QMap<int, ToolbarItem> toolbars;
-
-	PsiActionList::ActionsType class2id( QString name ) {
+	PsiActionList::ActionsType class2id() {
 		int ret = (int)PsiActionList::Actions_Common;
-
-		if ( name == "mainWin" )
-			ret |= (int)PsiActionList::Actions_MainWin;
-
+		ret |= (int)PsiActionList::Actions_MainWin;
 		return (PsiActionList::ActionsType)ret;
 	}
 };
 
-OptionsTabToolbars::OptionsTabToolbars(QObject *parent) 
-: OptionsTab(parent, "toolbars", "", tr("Toolbars"), tr("Configure Psi toolbars"), "psi/toolbars")
+OptionsTabToolbars::OptionsTabToolbars(QObject *parent)
+	: OptionsTab(parent, "toolbars", "", tr("Toolbars"), tr("Configure Psi toolbars"), "psi/toolbars")
 {
 	w = 0;
 	p = new Private();
 
 	noDirty = false;
-	opt = new Options;
 }
 
 QWidget *OptionsTabToolbars::widget()
 {
 	if (w)
 		return 0;
-	
+
 	w = new LookFeelToolbarsUI();
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI*) w;
-	
+
 	connect(d->pb_addToolbar, SIGNAL(clicked()), SLOT(toolbarAdd()));
 	connect(d->pb_deleteToolbar, SIGNAL(clicked()), SLOT(toolbarDelete()));
 	connect(d->cb_toolbars, SIGNAL(activated(int)), SLOT(toolbarSelectionChanged(int)));
 	connect(d->le_toolbarName, SIGNAL(textChanged(const QString &)), SLOT(toolbarNameChanged()));
-	connect(d->pb_toolbarPosition, SIGNAL(clicked()), SLOT(toolbarPosition()));
+	// connect(d->pb_toolbarPosition, SIGNAL(clicked()), SLOT(toolbarPosition()));
 	connect(d->tb_up, SIGNAL(clicked()), SLOT(toolbarActionUp()));
 	connect(d->tb_down, SIGNAL(clicked()), SLOT(toolbarActionDown()));
 	connect(d->tb_right, SIGNAL(clicked()), SLOT(toolbarAddAction()));
@@ -200,7 +77,7 @@ QWidget *OptionsTabToolbars::widget()
 
 	connect(d->ck_toolbarOn, SIGNAL(toggled(bool)), SLOT(toolbarDataChanged()));
 	connect(d->ck_toolbarLocked, SIGNAL(toggled(bool)), SLOT(toolbarDataChanged()));
-	connect(d->ck_toolbarStretch, SIGNAL(toggled(bool)), SLOT(toolbarDataChanged()));
+	// connect(d->ck_toolbarStretch, SIGNAL(toggled(bool)), SLOT(toolbarDataChanged()));
 	connect(d->lw_selectedActions, SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *)), SLOT(selAct_selectionChanged(QListWidgetItem *)));
 	connect(d->tw_availActions, SIGNAL(currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)), SLOT(avaAct_selectionChanged(QTreeWidgetItem *)));
 
@@ -253,7 +130,6 @@ QWidget *OptionsTabToolbars::widget()
 
 OptionsTabToolbars::~OptionsTabToolbars()
 {
-	delete opt;
 	delete p;
 }
 
@@ -289,65 +165,55 @@ void OptionsTabToolbars::setData(PsiCon * psi_, QWidget *parent_)
 	}
 }*/
 
-void OptionsTabToolbars::applyOptions(Options *o)
+void OptionsTabToolbars::applyOptions()
 {
-	if ( !w )
-		opt->toolbars = o->toolbars;
-
-	// get current toolbars' positions
-	QList<PsiToolBar*> toolbars = psi->toolbarList();
-	for (int i = 0; i < opt->toolbars["mainWin"].count() && i < toolbars.count(); i++) {
-		//if ( toolbarPositionInProgress && posTbDlg->n() == (int)i )
-		//	continue;
-
-		Options::ToolbarPrefs &tbPref = opt->toolbars["mainWin"][i];
-		psi->getToolbarLocation(toolbars.at(i), tbPref.dock, tbPref.index, tbPref.nl, tbPref.extraOffset);
-	}
-
-	// apply options
-	o->toolbars = opt->toolbars;
-
-	doApply();
-}
-
-void OptionsTabToolbars::rebuildToolbarList()
-{
-	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
-
-	d->cb_toolbars->clear();
-	p->toolbars.clear();
-
-	QMap< QString, QList<Options::ToolbarPrefs> >::Iterator it = opt->toolbars.begin();
-	for ( ; it != opt->toolbars.end(); ++it ) {
-		if ( p->toolbars.count() )
-			d->cb_toolbars->insertItem( "---" ); // TODO: think of better separator
-
-		int i = 0;
-		QList<Options::ToolbarPrefs>::Iterator it2 = it.data().begin();
-		for ( ; it2 != it.data().end(); ++it2, ++i ) {
-			d->cb_toolbars->insertItem( (*it2).name );
-			p->toolbars[d->cb_toolbars->count()-1] = Private::ToolbarItem( it.key(), i );
-		}
-	}
-}
-
-void OptionsTabToolbars::restoreOptions(const Options *o)
-{
-	if ( !w )
+	if (!w)
 		return;
 
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
-	opt->toolbars = o->toolbars;
-	opt->hideMenubar = o->hideMenubar;
 
-	rebuildToolbarList();
+	PsiOptions *o = PsiOptions::instance();
+	o->removeOption("options.ui.contactlist.toolbars", true);
+	QMap<QString, ToolbarPrefs>::Iterator it = p->toolbars.begin();
+	for (; it != p->toolbars.end(); ++it) {
+		PsiToolBar::structToOptions(o, it.data());
+	}
+}
 
-	if(d->cb_toolbars->count() > 0) {
-		d->cb_toolbars->setCurrentIndex( 0 );
-		toolbarSelectionChanged( 0 );
+void OptionsTabToolbars::restoreOptions()
+{
+	if (!w)
+		return;
+
+	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
+
+	PsiOptions *o = PsiOptions::instance();
+
+	QStringList toolbarBases = o->getChildOptionNames("options.ui.contactlist.toolbars", true, true);
+
+	foreach(QString base, toolbarBases) {
+		ToolbarPrefs tb;
+		tb.id = o->getOption(base + ".key").toString();
+		tb.name = o->getOption(base + ".name").toString();
+		tb.on = o->getOption(base + ".visible").toBool();
+		tb.locked = o->getOption(base + ".locked").toBool();
+		// tb.stretchable = o->getOption(base + ".stretchable").toBool();
+		tb.dock = (Qt::Dock)o->getOption(base + ".dock.position").toInt(); //FIXME
+		// tb.index = o->getOption(base + ".dock.index").toInt();
+		tb.nl = o->getOption(base + ".dock.nl").toBool();
+		// tb.extraOffset = o->getOption(base + ".dock.extra-offset").toInt();
+		tb.keys = o->getOption(base + ".actions").toStringList();
+
+		p->toolbars[base] = tb;
+		d->cb_toolbars->addItem(tb.name, base);
+	}
+
+	if (d->cb_toolbars->count() > 0) {
+		d->cb_toolbars->setCurrentIndex(0);
+		toolbarSelectionChanged(0);
 	}
 	else
-		toolbarSelectionChanged( -1 );
+		toolbarSelectionChanged(-1);
 }
 
 //----------------------------------------------------------------------------
@@ -358,25 +224,47 @@ void OptionsTabToolbars::toolbarAdd()
 
 	int i = d->cb_toolbars->count();
 
-	Options::ToolbarPrefs tb;
-	tb.name = QObject::tr("<unnamed>");
+	ToolbarPrefs tb;
+	int j = 0;
+	bool ok;
+	do {
+		ok = true;
+		tb.name = QObject::tr("<unnamed%1>").arg(j++);
+		foreach(ToolbarPrefs other, p->toolbars) {
+			if (other.name == tb.name) {
+				ok = false;
+				break;
+			}
+		}
+	}
+	while (!ok);
+
 	tb.on = false;
 	tb.locked = false;
-	tb.stretchable = false;
+	// tb.stretchable = false;
 	tb.keys.clear();
 
 	tb.dock = Qt::DockTop;
-	tb.index = i;
+	// tb.index = i;
 	tb.nl = true;
-	tb.extraOffset = 0;
+	// tb.extraOffset = 0;
 
 	tb.dirty = true;
 
-	opt->toolbars["mainWin"].append(tb);
+	QString base;
+	j = 0;
+	do {
+		ok = true;
+		base = ".." + QString::number(j++);
+	}
+	while (p->toolbars.keys().contains(base));
 
-	rebuildToolbarList();
-	d->cb_toolbars->setCurrentIndex( d->cb_toolbars->count()-1 );
-	toolbarSelectionChanged( d->cb_toolbars->currentIndex() );
+	p->toolbars[base] = tb;
+
+	d->cb_toolbars->addItem(tb.name, base);
+
+	d->cb_toolbars->setCurrentIndex(d->cb_toolbars->count() - 1);
+	toolbarSelectionChanged(d->cb_toolbars->currentIndex());
 
 	d->le_toolbarName->setFocus();
 }
@@ -386,27 +274,24 @@ void OptionsTabToolbars::toolbarDelete()
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
 	int n = d->cb_toolbars->currentIndex();
 
-	{
-		int idx = p->toolbars[n].index;
-		QList<Options::ToolbarPrefs>::Iterator it = opt->toolbars[p->toolbars[n].group].begin();
-		for (int i = 0; i < idx; i++)
-			++it;
+	QString base = d->cb_toolbars->itemData(n).toString();
 
-		noDirty = true;
-		opt->toolbars[p->toolbars[n].group].remove(it);
+	noDirty = true;
+	toolbarSelectionChanged(-1);
 
-		toolbarSelectionChanged(-1);
-		rebuildToolbarList();
-		noDirty = false;
-		toolbarSelectionChanged( d->cb_toolbars->currentIndex() );
-	}
+	p->toolbars.remove(base);
+
+	d->cb_toolbars->removeItem(d->cb_toolbars->findData(base));
+
+	noDirty = false;
+	toolbarSelectionChanged(d->cb_toolbars->currentIndex());
 }
 
 void OptionsTabToolbars::addToolbarAction(QListWidget *parent, QString name, int toolbarId)
 {
-	ActionList actions = psi->actionList()->suitableActions( (PsiActionList::ActionsType)toolbarId );
-	const QAction *action = (QAction *)actions.action( name );
-	if ( !action )
+	ActionList actions = psi->actionList()->suitableActions((PsiActionList::ActionsType)toolbarId);
+	const QAction *action = (QAction *)actions.action(name);
+	if (!action)
 		return;
 	addToolbarAction(parent, action, name);
 }
@@ -416,92 +301,99 @@ void OptionsTabToolbars::addToolbarAction(QListWidget *parent, const QAction *ac
 	QListWidgetItem *item = new QListWidgetItem(parent);
 
 	QString n = actionName(action);
-	if ( !action->whatsThis().isEmpty() )
+	if (!action->whatsThis().isEmpty())
 		n += " - " + action->whatsThis();
 	item->setText(n);
 	item->setData(Qt::UserRole, name);
 	item->setIcon(action->iconSet());
+	item->setHidden(!action->isVisible());
 }
 
 void OptionsTabToolbars::toolbarSelectionChanged(int item)
 {
-	if ( noDirty )
+	if (noDirty)
 		return;
 
 	int n = item;
-	PsiToolBar *toolBar = 0;
-	if ( item != -1 )
-		toolBar = psi->findToolBar( p->toolbars[n].group, p->toolbars[n].index );
+//	PsiToolBar *toolBar = 0;
+//	if ( item != -1 )
+//		toolBar = psi->findToolBar( p->toolbars[n].group, p->toolbars[n].index );
 
-	bool customizeable = toolBar ? toolBar->isCustomizeable() : true;
-	bool moveable      = toolBar ? toolBar->isMoveable() : true;
+	bool customizeable = true;
+	bool moveable      = true;
 
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
-	bool enable = (item == -1) ? false: true;
-	d->le_toolbarName->setEnabled( enable );
-	d->pb_toolbarPosition->setEnabled( enable && moveable );
-	d->ck_toolbarOn->setEnabled( enable );
-	d->ck_toolbarLocked->setEnabled( enable && moveable );
-	d->ck_toolbarStretch->setEnabled( enable && moveable );
-	d->lw_selectedActions->setEnabled( enable && customizeable );
-	d->tw_availActions->setEnabled( enable && customizeable );
-	d->tb_up->setEnabled( enable && customizeable );
-	d->tb_down->setEnabled( enable && customizeable );
-	d->tb_left->setEnabled( enable && customizeable );
-	d->tb_right->setEnabled( enable && customizeable );
-	d->pb_deleteToolbar->setEnabled( enable && p->toolbars[n].group == "mainWin" );
-	d->cb_toolbars->setEnabled( enable );
+	bool enable = (item == -1) ? false : true;
+	d->le_toolbarName->setEnabled(enable);
+	// d->pb_toolbarPosition->setEnabled(enable && moveable);
+	d->ck_toolbarOn->setEnabled(enable);
+	d->ck_toolbarLocked->setEnabled(enable && moveable);
+	// d->ck_toolbarStretch->setEnabled(enable && moveable);
+	d->lw_selectedActions->setEnabled(enable && customizeable);
+	d->tw_availActions->setEnabled(enable && customizeable);
+	d->tb_up->setEnabled(enable && customizeable);
+	d->tb_down->setEnabled(enable && customizeable);
+	d->tb_left->setEnabled(enable && customizeable);
+	d->tb_right->setEnabled(enable && customizeable);
+	d->pb_deleteToolbar->setEnabled(enable);
+	d->cb_toolbars->setEnabled(enable);
 
 	d->tw_availActions->clear();
 	d->lw_selectedActions->clear();
 
-	if ( !enable ) {
-		d->le_toolbarName->setText( "" );
+	if (!enable) {
+		d->le_toolbarName->setText("");
 		return;
 	}
 
 	noDirty = true;
 
-	Options::ToolbarPrefs tb = opt->toolbars[p->toolbars[n].group][p->toolbars[n].index];
-	d->le_toolbarName->setText( tb.name );
-	d->ck_toolbarOn->setChecked( tb.on );
-	d->ck_toolbarLocked->setChecked( tb.locked || !moveable );
-	d->ck_toolbarStretch->setChecked( tb.stretchable );
+	QString base = d->cb_toolbars->itemData(n).toString();
+	ToolbarPrefs tb;
+	tb = p->toolbars[base];
+
+	d->le_toolbarName->setText(tb.name);
+	d->ck_toolbarOn->setChecked(tb.on);
+	d->ck_toolbarLocked->setChecked(tb.locked || !moveable);
+	// d->ck_toolbarStretch->setChecked(tb.stretchable);
 
 	{
 		// Fill the TreeWidget with toolbar-specific actions
 		QTreeWidget *tw = d->tw_availActions;
 		QTreeWidgetItem *lastRoot = 0;
 
-		foreach(ActionList* actionList, psi->actionList()->actionLists( p->class2id( p->toolbars[n].group ) )) {
+		foreach(ActionList* actionList, psi->actionList()->actionLists(p->class2id())) {
 			QTreeWidgetItem *root = new QTreeWidgetItem(tw, lastRoot);
 			lastRoot = root;
-			root->setText( 0, actionList->name() );
-			root->setData( 0, Qt::UserRole, QString("") );
-			root->setExpanded( true );
+			root->setText(0, actionList->name());
+			root->setData(0, Qt::UserRole, QString(""));
+			root->setExpanded(true);
 
 			QTreeWidgetItem *last = 0;
 			QStringList actionNames = actionList->actions();
 			QStringList::Iterator it2 = actionNames.begin();
-			for ( ; it2 != actionNames.end(); ++it2 ) {
-				IconAction *action = actionList->action( *it2 );
-		        	QTreeWidgetItem *item = new QTreeWidgetItem( root, last );
+			for (; it2 != actionNames.end(); ++it2) {
+				IconAction *action = actionList->action(*it2);
+				if (!action->isVisible())
+					continue;
+				QTreeWidgetItem *item = new QTreeWidgetItem(root, last);
 				last = item;
 
 				QString n = actionName((QAction *)action);
-				if ( !action->whatsThis().isEmpty() )
+				if (!action->whatsThis().isEmpty()) {
 					n += " - " + action->whatsThis();
-				item->setText( 0, n );
-				item->setIcon( 0, action->iconSet() );
-				item->setData( 0, Qt::UserRole, action->name() );
+				}
+				item->setText(0, n);
+				item->setIcon(0, action->iconSet());
+				item->setData(0, Qt::UserRole, action->name());
 			}
 		}
 		tw->resizeColumnToContents(0);
 	}
 
 	QStringList::Iterator it = tb.keys.begin();
-	for ( ; it != tb.keys.end(); ++it) {
-		addToolbarAction(d->lw_selectedActions, *it, p->class2id( p->toolbars[n].group ) );
+	for (; it != tb.keys.end(); ++it) {
+		addToolbarAction(d->lw_selectedActions, *it, p->class2id());
 	}
 	updateArrows();
 
@@ -511,7 +403,7 @@ void OptionsTabToolbars::toolbarSelectionChanged(int item)
 void OptionsTabToolbars::rebuildToolbarKeys()
 {
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
-	if ( !d->cb_toolbars->count() )
+	if (!d->cb_toolbars->count())
 		return;
 	int n = d->cb_toolbars->currentIndex();
 
@@ -520,10 +412,11 @@ void OptionsTabToolbars::rebuildToolbarKeys()
 	int count = d->lw_selectedActions->count();
 	for (int i = 0; i < count; i++) {
 		keys << d->lw_selectedActions->item(i)->data(Qt::UserRole).toString();
-        }
+	}
+
+	QString base = d->cb_toolbars->itemData(n).toString();
+	p->toolbars[base].keys = keys;
 
-	opt->toolbars["mainWin"][n].keys  = keys;
-	opt->toolbars["mainWin"][n].dirty = true;
 	emit dataChanged();
 }
 
@@ -532,19 +425,30 @@ void OptionsTabToolbars::updateArrows()
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
 	bool up = false, down = false, left = false, right = false;
 
-	if(d->tw_availActions->currentItem() && !d->tw_availActions->currentItem()->data(0, Qt::UserRole).toString().isEmpty())
+	if (d->tw_availActions->currentItem() && !d->tw_availActions->currentItem()->data(0, Qt::UserRole).toString().isEmpty())
 		right = true;
 	QListWidgetItem *i = d->lw_selectedActions->currentItem();
-	if(i) {
+	if (i) {
 		left = true;
 
 		// get numeric index of item
 		int n = d->lw_selectedActions->row(i);
 
-		if(n > 0)
-			up = true;
-		if(n < d->lw_selectedActions->count() - 1)
-			down = true;
+		int i = n;
+		while (--i > 0) {
+			if (!d->lw_selectedActions->item(i)->isHidden()) {
+				up = true;
+				break;
+			}
+		}
+
+		i = n;
+		while (++i < d->lw_selectedActions->count()) {
+			if (!d->lw_selectedActions->item(i)->isHidden()) {
+				down = true;
+				break;
+			}
+		}
 	}
 
 	d->tb_up->setEnabled(up);
@@ -556,12 +460,16 @@ void OptionsTabToolbars::updateArrows()
 void OptionsTabToolbars::toolbarNameChanged()
 {
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
-	if ( !d->cb_toolbars->count() )
+	if (!d->cb_toolbars->count())
 		return;
 
+	QString name = d->le_toolbarName->text();
+
 	int n = d->cb_toolbars->currentIndex();
-	d->cb_toolbars->changeItem(d->le_toolbarName->text(), n);
-	opt->toolbars["mainWin"][n].name = d->le_toolbarName->text();
+	QString base = d->cb_toolbars->itemData(n).toString();
+	p->toolbars[base].name = name;
+
+	d->cb_toolbars->setItemText(d->cb_toolbars->findData(base), name);
 
 	emit dataChanged();
 }
@@ -570,13 +478,17 @@ void OptionsTabToolbars::toolbarActionUp()
 {
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
 	QListWidgetItem *item = d->lw_selectedActions->currentItem();
-	if ( !item )
+	if (!item)
 		return;
 
 	int row = d->lw_selectedActions->row(item);
-	if ( row > 0 ) {
+	if (row > 0) {
 		d->lw_selectedActions->takeItem(row);
-		d->lw_selectedActions->insertItem(row - 1, item);
+		--row;
+		while (row > 0 && d->lw_selectedActions->item(row)->isHidden()) {
+			--row;
+		}
+		d->lw_selectedActions->insertItem(row, item);
 		d->lw_selectedActions->setCurrentItem(item);
 	}
 
@@ -588,13 +500,17 @@ void OptionsTabToolbars::toolbarActionDown()
 {
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
 	QListWidgetItem *item = d->lw_selectedActions->currentItem();
-	if ( !item )
+	if (!item)
 		return;
 
 	int row = d->lw_selectedActions->row(item);
-	if ( row < d->lw_selectedActions->count() ) {
+	if (row < d->lw_selectedActions->count()) {
 		d->lw_selectedActions->takeItem(row);
-		d->lw_selectedActions->insertItem(row + 1, item);
+		++row;
+		while (row < d->lw_selectedActions->count() && d->lw_selectedActions->item(row)->isHidden()) {
+			++row;
+		}
+		d->lw_selectedActions->insertItem(row, item);
 		d->lw_selectedActions->setCurrentItem(item);
 	}
 
@@ -606,10 +522,10 @@ void OptionsTabToolbars::toolbarAddAction()
 {
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
 	QTreeWidgetItem *item = d->tw_availActions->currentItem();
-	if ( !item || item->data(0, Qt::UserRole).toString().isEmpty() )
+	if (!item || item->data(0, Qt::UserRole).toString().isEmpty())
 		return;
 
-	addToolbarAction(d->lw_selectedActions, item->data(0, Qt::UserRole).toString(), p->class2id( p->toolbars[d->cb_toolbars->currentIndex()].group ) );
+	addToolbarAction(d->lw_selectedActions, item->data(0, Qt::UserRole).toString(), p->class2id());
 	rebuildToolbarKeys();
 	updateArrows();
 }
@@ -618,7 +534,7 @@ void OptionsTabToolbars::toolbarRemoveAction()
 {
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
 	QListWidgetItem *item = d->lw_selectedActions->currentItem();
-	if ( !item )
+	if (!item)
 		return;
 
 	delete item;
@@ -629,19 +545,25 @@ void OptionsTabToolbars::toolbarRemoveAction()
 
 void OptionsTabToolbars::toolbarDataChanged()
 {
-	if ( noDirty )
+	if (noDirty)
 		return;
 
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
-	if ( !d->cb_toolbars->count() )
+	if (!d->cb_toolbars->count())
 		return;
 	int n = d->cb_toolbars->currentIndex();
 
-	opt->toolbars["mainWin"][n].dirty = true;
-	opt->toolbars["mainWin"][n].name = d->le_toolbarName->text();
-	opt->toolbars["mainWin"][n].on = d->ck_toolbarOn->isChecked();
-	opt->toolbars["mainWin"][n].locked = d->ck_toolbarLocked->isChecked();
-	opt->toolbars["mainWin"][n].stretchable = d->ck_toolbarStretch->isChecked();
+	QString base = d->cb_toolbars->itemData(n).toString();
+	ToolbarPrefs tb;
+	tb = p->toolbars[base];
+
+	tb.dirty = true;
+	tb.name = d->le_toolbarName->text();
+	tb.on = d->ck_toolbarOn->isChecked();
+	tb.locked = d->ck_toolbarLocked->isChecked();
+	// tb.stretchable = d->ck_toolbarStretch->isChecked();
+
+	p->toolbars[base] = tb;
 
 	emit dataChanged();
 }
@@ -650,9 +572,9 @@ QString OptionsTabToolbars::actionName(const QAction *a)
 {
 	QString n = a->menuText(), n2;
 	for (int i = 0; i < (int)n.length(); i++) {
-		if ( n[i] == '&' && n[i+1] != '&' )
+		if (n[i] == '&' && n[i+1] != '&')
 			continue;
-		else if ( n[i] == '&' && n[i+1] == '&' )
+		else if (n[i] == '&' && n[i+1] == '&')
 			n2 += '&';
 		else
 			n2 += n[i];
@@ -663,30 +585,30 @@ QString OptionsTabToolbars::actionName(const QAction *a)
 
 void OptionsTabToolbars::toolbarPosition()
 {
+#if 0
+	LEGOPTFIXME
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
-	if ( !d->cb_toolbars->count() )
+	if (!d->cb_toolbars->count())
 		return;
 	int n = d->cb_toolbars->currentIndex();
 
-	PositionOptionsTabToolbars *posTbDlg = new PositionOptionsTabToolbars(w, &opt->toolbars["mainWin"][n], n);
+	PositionOptionsTabToolbars *posTbDlg = new PositionOptionsTabToolbars(w, &LEGOPTS.toolbars["mainWin"][n], n);
 	connect(posTbDlg, SIGNAL(applyPressed()), SLOT(toolbarPositionApply()));
 
 	posTbDlg->exec();
 	delete posTbDlg;
+#endif
 }
 
 void OptionsTabToolbars::toolbarPositionApply()
 {
+#if 0
+	LEGOPTFIXME
 	emit dataChanged();
 
-	option.toolbars = opt->toolbars;
-	psi->buildToolbars();
-}
-
-void OptionsTabToolbars::doApply()
-{
-	option.toolbars = opt->toolbars;
+	LEGOPTS.toolbars = LEGOPTS.toolbars;
 	psi->buildToolbars();
+#endif
 }
 
 void OptionsTabToolbars::selAct_selectionChanged(QListWidgetItem *)
@@ -698,5 +620,3 @@ void OptionsTabToolbars::avaAct_selectionChanged(QTreeWidgetItem *)
 {
 	updateArrows();
 }
-
-#include "opt_toolbars.moc"
