@@ -30,6 +30,7 @@
 
 #include "xmpp_rosterx.h"
 #include "xmpp_status.h"
+#include "psiactions.h"
 
 namespace XMPP
 {
@@ -59,6 +60,7 @@ class BookmarkManager;
 class URLBookmark;
 class ConferenceBookmark;
 class VoiceCaller;
+class CapsRegistry;
 class UserAccount;
 class ContactProfile;
 class QWidget;
@@ -77,6 +79,7 @@ class QHostAddress;
 class AvatarFactory;
 class PEPManager;
 class ServerInfoManager;
+class TabManager;
 #ifdef GOOGLE_FT
 class GoogleFileTransfer;
 #endif
@@ -88,12 +91,13 @@ class PsiAccount : public QObject
 {
 	Q_OBJECT
 public:
-	PsiAccount(const UserAccount &acc, PsiContactList *parent);
+	PsiAccount(const UserAccount &acc, PsiContactList *parent, CapsRegistry* capsRegistry, TabManager *tabManager);
 	~PsiAccount();
 
 	bool enabled() const;
 	void setEnabled(bool e = TRUE);
 
+	bool isAvailable() const;
 	bool isActive() const;
 	bool isConnected() const;
 	const QString &name() const;
@@ -123,6 +127,8 @@ public:
 	bool hasPGP() const;
 	QHostAddress *localAddress() const;
 
+	ChatDlg* findChatDialog(const Jid& jid) const;
+
 	template<typename T>
 	inline T findDialog(const Jid& jid = Jid(), bool compareResource = true) const { 
 		return static_cast<T>(findDialog(((T)0)->staticMetaObject, jid, compareResource));
@@ -147,7 +153,7 @@ public:
 
 	void showXmlConsole();
 	void openAddUserDlg();
-	void openGroupChat(const Jid &);
+	void openGroupChat(const Jid &, ActivationType activationType);
 	bool groupChatJoin(const QString &host, const QString &room, const QString &nick, const QString& pass, bool nohistory = false);
 	void groupChatSetStatus(const QString &host, const QString &room, const Status &);
 	void groupChatChangeNick(const QString &host, const QString &room, const QString& nick, const Status &);
@@ -182,6 +188,10 @@ public:
 	ServerInfoManager* serverInfoManager();
 	BookmarkManager* bookmarkManager();
 
+	enum xmlRingType {RingXmlIn, RingXmlOut, RingSysMsg};
+	class xmlRingElem { public: int type; QDateTime time; QString xml; };
+	QList< xmlRingElem > dumpRingbuf();
+
 signals:
 	void disconnected();
 	void reconnecting();
@@ -193,8 +203,9 @@ signals:
 	void updateContact(const Jid &, bool);
 	void nickChanged();
 	void pgpKeyChanged();
-	void encryptedMessageSent(int, bool, int);
+	void encryptedMessageSent(int, bool, int, const QString &);
 	void enabledChanged();
+	void startBounce();
 
 public slots:
 	void setStatus(const XMPP::Status &, bool withStatus = false);
@@ -206,7 +217,7 @@ public slots:
 	void incomingVoiceCall(const Jid&);
 	
 	void secondsIdle(int);
-	void openNextEvent();
+	void openNextEvent(ActivationType activationType);
 	int forwardPendingEvents(const Jid &jid);
 	void autoLogin();
 
@@ -257,7 +268,8 @@ public slots:
 	void actionHistoryBox(PsiEvent *);
 	void actionRegister(const Jid &);
 	void actionSearch(const Jid &);
-	void actionJoin(const Jid &, const QString & = QString());
+	void actionJoin(const Jid& mucJid, const QString& password = QString());
+	void actionJoin(const ConferenceBookmark& bookmark, bool connectImmediately);
 	void actionDisco(const Jid &, const QString &);
 	void actionInvite(const Jid &, const QString &);
 	void actionVoice(const Jid&);
@@ -312,7 +324,7 @@ private slots:
 	void serverFeaturesChanged();
 	void setPEPAvailable(bool);
 
-	void getBookmarks_success(const QList<URLBookmark>&, const QList<ConferenceBookmark>&);
+	void bookmarksAvailabilityChanged();
 
 	void incomingHttpAuthRequest(const PsiHttpAuthRequest &);
 
@@ -351,7 +363,10 @@ protected:
 	void sessionStarted();
 
 private slots:
-	void handleEvent(PsiEvent *);
+	void eventFromXml(PsiEvent* e);
+
+private:
+	void handleEvent(PsiEvent* e, ActivationType activationType);
 
 public:
 	class Private;
@@ -364,8 +379,8 @@ private:
 	void simulateRosterOffline();
 	void cpUpdate(const UserListItem &, const QString &rname="", bool fromPresence=false);
 	void logEvent(const Jid &, PsiEvent *);
-	void queueEvent(PsiEvent *);
-	void openNextEvent(const UserListItem &);
+	void queueEvent(PsiEvent* e, ActivationType activationType);
+	void openNextEvent(const UserListItem &, ActivationType activationType);
 	void updateReadNext(const Jid &);
 	ChatDlg *ensureChatDlg(const Jid &);
 	void lastStepLogin();
@@ -377,11 +392,11 @@ private:
 	void verifyStatus(const Jid &j, const Status &s);
 
 	void processChats(const Jid &);
-	void openChat(const Jid &);
+	void openChat(const Jid &, ActivationType activationType);
 	EventDlg *ensureEventDlg(const Jid &);
 	friend class PsiCon;
 
-	bool isDisconnecting, notifyOnlineOk, doReconnect, usingAutoStatus, rosterDone, presenceSent, v_isActive;
+	bool isDisconnecting, notifyOnlineOk, doReconnect, rosterDone, presenceSent, v_isActive;
 	void cleanupStream();
 
 	QWidget* findDialog(const QMetaObject& mo, const Jid& jid, bool compareResource) const;
