@@ -25,6 +25,10 @@
 #include <QList>
 
 #include "profiles.h"
+#include "psiactions.h"
+
+#include "tabbablewidget.h"
+
 
 using namespace XMPP;
 
@@ -74,33 +78,24 @@ public:
 	ProxyManager *proxy() const;
 	FileTransDlg *ftdlg() const;
 
-	TabDlg* getTabs();
-	TabDlg*	newTabs();
-	bool isChatTabbed(ChatDlg*);
-	bool isChatActiveWindow(ChatDlg*);
-	ChatDlg* getChatInTabs(QString);
-	TabDlg* getManagingTabs(ChatDlg*);
-	Q3PtrList<TabDlg>* getTabSets();
-
 	QWidget *dialogFind(const char *className);
 	void dialogRegister(QWidget *w);
 	void dialogUnregister(QWidget *w);
 
 	QMenuBar* defaultMenuBar() const;
 
-	void createAccount(const QString &name, const Jid &j="", const QString &pass="", bool opt_host=false, const QString &host="", int port=5222, bool legacy_ssl_probe = true, UserAccount::SSLFlag ssl=UserAccount::SSL_Auto, int proxy=0);
+	void createAccount(const QString &name, const Jid &j="", const QString &pass="", bool opt_host=false, const QString &host="", int port=5222, bool legacy_ssl_probe = true, UserAccount::SSLFlag ssl=UserAccount::SSL_Auto, QString proxy="");
 	PsiAccount *createAccount(const UserAccount &);
 	//void createAccount(const QString &, const QString &host="", int port=5222, bool ssl=false, const QString &user="", const QString &pass="");
 	void removeAccount(PsiAccount *);
 
 	void playSound(const QString &);
-	void raiseMainwin();
 
 	AccountsComboBox *accountsComboBox(QWidget *parent=0, bool online_only = false);
 
-	const QStringList & recentGCList() const;
+	QStringList recentGCList() const;
 	void recentGCAdd(const QString &);
-	const QStringList & recentBrowseList() const;
+	QStringList recentBrowseList() const;
 	void recentBrowseAdd(const QString &);
 	const QStringList & recentNodeList() const;
 	void recentNodeAdd(const QString &);
@@ -108,15 +103,10 @@ public:
 	EventDlg *createEventDlg(const QString &, PsiAccount *);
 	void updateContactGlobal(PsiAccount *, const Jid &);
 
-	QList<PsiToolBar*> toolbarList() const;
-	PsiToolBar *findToolBar(QString group, int index);
 	PsiActionList *actionList() const;
 
-	void buildToolbars();
-	bool getToolbarLocation(Q3DockWindow* dw, Qt::Dock& dock, int& index, bool& nl, int& extraOffset) const;
-
 	IconSelectPopup *iconSelectPopup() const;
-	void processEvent(PsiEvent *);
+	void processEvent(PsiEvent*, ActivationType activationType);
 
 signals:
 	void quit(int);
@@ -145,19 +135,21 @@ public slots:
 	void statusMenuChanged(int);
 	void pa_updatedActivity();
 	void pa_updatedAccount();
-	void slotApplyOptions(const Options &);
+	void slotApplyOptions();
 	void queueChanged();
 	void recvNextEvent();
 	void setStatusFromDialog(const XMPP::Status &, bool withPriority);
 	void proxy_settingsChanged();
 	void updateMainwinStatus();
-	void tabDying(TabDlg*);
-
-	void mainWinGeomChanged(QRect saveableGeometry);
+	void doOpenUri(const QUrl &uri);
+	void raiseMainwin();
 
 private slots:
 	void saveAccounts();
-	void optionsUpdate();
+	void saveCapabilities();
+	void optionChanged(const QString& option);
+	void forceSavePreferences();
+	void startBounce();
 
 private:
 	class Private;
@@ -169,9 +161,10 @@ private:
 	void setShortcuts();
 
 	friend class PsiAccount; // FIXME
-	void setToggles(bool tog_offline, bool tog_away, bool tog_agents, bool tog_hidden, bool tog_self);
-	void getToggles(bool *tog_offline, bool *tog_away, bool *tog_agents, bool *tog_hidden, bool *tog_self);
 	void promptUserToCreateAccount();
+	QString optionsFile() const;
+
+	void registerCaps(const QString& ext, const QStringList& features);
 
 	friend class EventQueue;
 	int getId();
