@@ -30,25 +30,22 @@
 #include "filetransfer.h"
 #include "applicationinfo.h"
 #include "psicontactlist.h"
+#include "atomicxmlfile.h"
+#include "psioptions.h"
+
+// FIXME renames those
+const int eventPriorityHeadline = 0;
+const int eventPriorityChat     = 1;
+const int eventPriorityMessage  = 1;
+const int eventPriorityAuth     = 2;
+//const int eventPriorityFile     = 3;
+const int eventPriorityFile     = 2;
+const int eventPriorityRosterExchange = 0; // LEGOPTFIXME: was uninitialised
+
 
 using namespace XMPP;
 using namespace XMLHelper;
 
-static PsiEvent *copyPsiEvent(PsiEvent *fe)
-{
-	PsiEvent *e = 0;
-	if ( fe->inherits("MessageEvent") )
-		e = new MessageEvent( *((MessageEvent *)fe) );
-	else if ( fe->inherits("AuthEvent") )
-		e = new AuthEvent( *((AuthEvent *)fe) );
-	else if ( fe->inherits("PGPEvent") )
-		e = new PGPEvent( *((PGPEvent *)fe) );
-	else
-		qWarning("copyPsiEvent(): Failed: unknown event type: %s", fe->className());
-
-	return 0;
-}
-
 //----------------------------------------------------------------------------
 // DummyStream
 //----------------------------------------------------------------------------
@@ -56,7 +53,7 @@ class DummyStream : public Stream
 {
 public:
 	QDomDocument & doc() const { return v_doc; }
-	QString baseNS() const { return QString::null; }
+	QString baseNS() const { return "jabber:client"; }
 	bool old() const { return false; }
 
 	void close() { }
@@ -190,7 +187,12 @@ bool PsiEvent::fromXml(PsiCon *psi, PsiAccount *account, const QDomElement *e)
 
 int PsiEvent::priority() const
 {
-	return Options::EventPriorityDontCare;
+	return EventPriorityDontCare;
+}
+
+QString PsiEvent::description() const
+{
+	return QString();
 }
 
 PsiEvent *PsiEvent::copy() const
@@ -291,8 +293,15 @@ bool MessageEvent::fromXml(PsiCon *psi, PsiAccount *account, const QDomElement *
 	QDomElement msg = findSubTag(*e, "message", &found);
 	if ( found ) {
 		DummyStream stream;
-		Stanza s = stream.createStanza(msg);
+		Stanza s = stream.createStanza(addCorrectNS(msg));
 		v_m.fromStanza(s, 0); // FIXME: fix tzoffset?
+
+		// if message was not spooled, it will be initialized with the
+		// current datetime. we want to reset it back to the original
+		// receive time
+		if (!v_m.timeStamp().secsTo(QDateTime::currentDateTime()))
+			v_m.setTimeStamp(timeStamp());
+
 		return true;
 	}
 
@@ -302,11 +311,27 @@ bool MessageEvent::fromXml(PsiCon *psi, PsiAccount *account, const QDomElement *
 int MessageEvent::priority() const
 {
 	if ( v_m.type() == "headline" )
-		return option.eventPriorityHeadline;
+		return eventPriorityHeadline;
 	else if ( v_m.type() == "chat" )
-		return option.eventPriorityChat;
+		return eventPriorityChat;
 
-	return option.eventPriorityMessage;
+	return eventPriorityMessage;
+}
+
+QString MessageEvent::description() const
+{
+	QStringList result;
+	if (!v_m.subject().isEmpty())
+		result << v_m.subject();
+	if (!v_m.body().isEmpty())
+		result << v_m.body();
+	foreach(Url url, v_m.urlList()) {
+		QString text = url.url();
+		if (!url.desc().isEmpty())
+			text += QString("(%1)").arg(url.desc());
+		result << text;
+	}
+	return result.join("\n");
 }
 
 PsiEvent *MessageEvent::copy() const
@@ -389,7 +414,22 @@ bool AuthEvent::fromXml(PsiCon *psi, PsiAccount *account, const QDomElement *e)
 
 int AuthEvent::priority() const
 {
-	return option.eventPriorityAuth;
+	return eventPriorityAuth;
+}
+
+QString AuthEvent::description() const
+{
+	QString result;
+	if (authType() == "subscribe")
+		result = tr("This user wants to subscribe to your presence.");
+	else if (authType() == "subscribed")
+		result = tr("You are now authorized.");
+	else if (authType() == "unsubscribed")
+		result = tr("Your authorization has been removed!");
+	else
+		Q_ASSERT(false);
+
+	return result;
 }
 
 PsiEvent *AuthEvent::copy() const
@@ -412,9 +452,16 @@ FileEvent::~FileEvent()
 	delete ft;
 }
 
+FileEvent::FileEvent(const FileEvent &from)
+: PsiEvent(from.account())
+{
+	v_from = from.v_from;
+	ft = from.ft->copy();
+}
+
 int FileEvent::priority() const
 {
-	return option.eventPriorityFile;
+	return eventPriorityFile;
 }
 
 Jid FileEvent::from() const
@@ -434,6 +481,16 @@ FileTransfer *FileEvent::takeFileTransfer()
 	return _ft;
 }
 
+QString FileEvent::description() const
+{
+	return tr("This user wants to send you a file.");
+}
+
+PsiEvent *FileEvent::copy() const
+{
+	return new FileEvent( *this );
+}
+
 //----------------------------------------------------------------------------
 // HttpAuthEvent
 //----------------------------------------------------------------------------
@@ -460,6 +517,12 @@ HttpAuthEvent::HttpAuthEvent(const PsiHttpAuthRequest &req, PsiAccount *acc)
 HttpAuthEvent::~HttpAuthEvent()
 {
 }
+
+QString HttpAuthEvent::description() const
+{
+	return tr("HTTP Authentication Request");
+}
+
 //----------------------------------------------------------------------------
 // RosterExchangeEvent
 //----------------------------------------------------------------------------
@@ -473,7 +536,7 @@ RosterExchangeEvent::RosterExchangeEvent(const Jid &j, const RosterExchangeItems
 
 int RosterExchangeEvent::priority() const
 {
-	return option.eventPriorityRosterExchange;
+	return eventPriorityRosterExchange;
 }
 
 Jid RosterExchangeEvent::from() const
@@ -506,6 +569,11 @@ void RosterExchangeEvent::setText(const QString& text)
 	v_text = text;
 }
 
+QString RosterExchangeEvent::description() const
+{
+	return tr("This user wants to modify your roster.");
+}
+
 //----------------------------------------------------------------------------
 // Status
 //----------------------------------------------------------------------------
@@ -518,7 +586,7 @@ void RosterExchangeEvent::setText(const QString& text)
 
 int StatusEvent::priority() const
 {
-	return option.eventPriorityChat;
+	return eventPriorityChat;
 }
 
 Jid StatusEvent::from() const
@@ -542,42 +610,38 @@ void StatusEvent::setStatus(const XMPP::Status& s)
 }*/
 
 //----------------------------------------------------------------------------
-// EventQueue
+// EventItem
 //----------------------------------------------------------------------------
 
-class EventItem
+EventItem::EventItem(PsiEvent *_e, int i)
 {
-public:
-        EventItem(PsiEvent *_e, int i)
-	{
-		e = _e;
-		v_id = i;
-	}
+	e = _e;
+	v_id = i;
+}
 
-	EventItem(const EventItem &from)
-	{
-		e = copyPsiEvent(from.e);
-		v_id = from.v_id;
-	}
+EventItem::EventItem(const EventItem &from)
+{
+	e = from.e->copy();
+	v_id = from.v_id;
+}
 
-	~EventItem()
-	{
-	}
+EventItem::~EventItem()
+{
+}
 
-	int id() const
-	{
-		return v_id;
-	}
+int EventItem::id() const
+{
+	return v_id;
+}
 
-	PsiEvent *event() const
-	{
-		return e;
-	}
+PsiEvent* EventItem::event() const
+{
+	return e;
+}
 
-private:
-	PsiEvent *e;
-	int v_id;
-};
+//----------------------------------------------------------------------------
+// EventQueue
+//----------------------------------------------------------------------------
 
 class EventQueue::Private
 {
@@ -668,12 +732,11 @@ void EventQueue::dequeue(PsiEvent *e)
 	foreach(EventItem *i, d->list) {
 		if ( e == i->event() ) {
 			d->list.remove(i);
+			emit queueChanged();
 			delete i;
 			return;
 		}
 	}
-
-	emit queueChanged();
 }
 
 PsiEvent *EventQueue::dequeue(const Jid &j, bool compareRes)
@@ -844,7 +907,7 @@ EventQueue &EventQueue::operator= (const EventQueue &from)
 
 	foreach(EventItem *i, from.d->list) {
 		PsiEvent *e = i->event();
-		enqueue( copyPsiEvent(e) );
+		enqueue( e->copy() );
 	}
 
 	return *this;
@@ -903,7 +966,7 @@ bool EventQueue::fromXml(const QDomElement *q)
 		}
 
 		if ( event )
-			emit handleEvent( event );
+			emit eventFromXml( event );
 	}
 
 	return true;
@@ -912,33 +975,19 @@ bool EventQueue::fromXml(const QDomElement *q)
 bool EventQueue::toFile(const QString &fname)
 {
 	QDomDocument doc;
-
 	QDomElement element = toXml(&doc);
 	doc.appendChild(element);
 
-	QFile f( fname );
-	if( !f.open(QIODevice::WriteOnly) )
-		return FALSE;
-	QTextStream t;
-	t.setDevice( &f );
-	t.setEncoding( QTextStream::UnicodeUTF8 );
-	t << doc.toString(4);
-	f.close();
-
-	return TRUE;
+	AtomicXmlFile f(fname);
+	return f.saveDocument(doc);
 }
 
 bool EventQueue::fromFile(const QString &fname)
 {
-	QString confver;
+	AtomicXmlFile f(fname);
 	QDomDocument doc;
-
-	QFile f(fname);
-	if(!f.open(QIODevice::ReadOnly))
-		return FALSE;
-	if(!doc.setContent(&f, true))
-		return FALSE;
-	f.close();
+	if (!f.loadDocument(&doc))
+		return false;
 
 	QDomElement base = doc.documentElement();
 	return fromXml(&base);
