@@ -116,7 +116,7 @@ void PsiPopup::Private::init(const PsiIcon *_titleIcon, QString titleText, PsiAc
 	account = acc;
 	display = true;
 
-	if ( !option.ppIsOn )
+	if ( !PsiOptions::instance()->getOption("options.ui.notifications.passive-popups.enabled").toBool() )
 		return;
 
 	if ( !psiPopupList )
@@ -134,8 +134,8 @@ void PsiPopup::Private::init(const PsiIcon *_titleIcon, QString titleText, PsiAc
 	else
 		titleIcon = new PsiIcon(*_titleIcon);
 
-	FancyPopup::setHideTimeout( option.ppHideTime );
-	FancyPopup::setBorderColor( option.ppBorderColor );
+	FancyPopup::setHideTimeout( PsiOptions::instance()->getOption("options.ui.notifications.passive-popups.duration").toInt() );
+	FancyPopup::setBorderColor( PsiOptions::instance()->getOption("options.ui.look.colors.passive-popup.border").value<QColor>() );
 
 	popup = new FancyPopup(titleText, titleIcon, lastPopup, false);
 	connect(popup, SIGNAL(clicked(int)), SLOT(popupClicked(int)));
@@ -157,7 +157,7 @@ void PsiPopup::Private::popupClicked(int button)
 {
 	if ( button == (int)Qt::LeftButton ) {
 		if ( event )
-			psi->processEvent( event );
+			psi->processEvent(event, UserAction);
 		else if ( account ) {
 			// FIXME: it should work in most cases, but
 			// maybe it's better to fix UserList::find()?
@@ -175,10 +175,10 @@ void PsiPopup::Private::eventDestroyed()
 
 QString PsiPopup::Private::clipText(QString text)
 {
-	if ( option.ppTextClip > 0 ) {
+	if ( PsiOptions::instance()->getOption("options.ui.notifications.passive-popups.maximum-text-length").toInt() > 0 ) {
 		// richtext will give us trouble here
-		if ( ((int)text.length()) > option.ppTextClip ) {
-			text = text.left( option.ppTextClip );
+		if ( ((int)text.length()) > PsiOptions::instance()->getOption("options.ui.notifications.passive-popups.maximum-text-length").toInt() ) {
+			text = text.left( PsiOptions::instance()->getOption("options.ui.notifications.passive-popups.maximum-text-length").toInt() );
 
 			// delete last unclosed tag
 			/*if ( text.find("</") > text.find(">") ) {
@@ -208,7 +208,7 @@ QBoxLayout *PsiPopup::Private::createContactInfo(const PsiIcon *icon, QString te
 
 	QLabel *textLabel = new QLabel(popup);
 	QFont font;
-	font.fromString( option.font[fPopup] );
+	font.fromString( PsiOptions::instance()->getOption("options.ui.look.font.passive-popup").toString() );
 	textLabel->setFont(font);
 
 	textLabel->setWordWrap(false);
@@ -318,15 +318,15 @@ void PsiPopup::setData(const Jid &j, const Resource &r, const UserListItem *u, c
 	QString text;
 
 	QString jid = j.full();
-	if ( option.ppJidClip > 0 && ((int)jid.length()) > option.ppJidClip )
-		jid = jid.left( option.ppJidClip ) + "...";
+	if ( PsiOptions::instance()->getOption("options.ui.notifications.passive-popups.maximum-jid-length").toInt() > 0 && ((int)jid.length()) > PsiOptions::instance()->getOption("options.ui.notifications.passive-popups.maximum-jid-length").toInt() )
+		jid = jid.left( PsiOptions::instance()->getOption("options.ui.notifications.passive-popups.maximum-jid-length").toInt() ) + "...";
 
 	QString status;
-	if ( option.ppStatusClip != 0 )
+	if ( PsiOptions::instance()->getOption("options.ui.notifications.passive-popups.maximum-status-length").toInt() != 0 )
 		status = r.status().status();
-	if ( option.ppStatusClip > 0 )
-		if ( ((int)status.length()) > option.ppStatusClip )
-			status = status.left ( option.ppStatusClip ) + "...";
+	if ( PsiOptions::instance()->getOption("options.ui.notifications.passive-popups.maximum-status-length").toInt() > 0 )
+		if ( ((int)status.length()) > PsiOptions::instance()->getOption("options.ui.notifications.passive-popups.maximum-status-length").toInt() )
+			status = status.left ( PsiOptions::instance()->getOption("options.ui.notifications.passive-popups.maximum-status-length").toInt() ) + "...";
 
 	QString name;
 	if ( u && !u->name().isEmpty() ) {
@@ -340,7 +340,7 @@ void PsiPopup::setData(const Jid &j, const Resource &r, const UserListItem *u, c
 	}
 		
 	if (!name.isEmpty()) {
-		if ( !option.ppJidClip )
+		if ( !PsiOptions::instance()->getOption("options.ui.notifications.passive-popups.maximum-jid-length").toInt() )
 			name = "<nobr>" + Qt::escape(name) + "</nobr>";
 		else
 			name = "<nobr>" + Qt::escape(name) + " &lt;" + Qt::escape(jid) + "&gt;" + "</nobr>";
@@ -349,7 +349,7 @@ void PsiPopup::setData(const Jid &j, const Resource &r, const UserListItem *u, c
 		name = "<nobr>&lt;" + Qt::escape(jid) + "&gt;</nobr>";
 
 	QString statusString = TextUtil::plain2rich(status);
-	if ( option.useEmoticons )
+	if ( PsiOptions::instance()->getOption("options.ui.emoticons.use-emoticons").toBool() )
 		statusString = TextUtil::emoticonify(statusString);
 	if( PsiOptions::instance()->getOption("options.ui.chat.legacy-formatting").toBool() )
 		statusString = TextUtil::legacyFormat(statusString);
@@ -371,7 +371,7 @@ void PsiPopup::setData(const Jid &j, const Resource &r, const UserListItem *u, c
 	}
 
 	// show popup
-	if ( d->popupType != AlertHeadline && (d->popupType != AlertFile || !option.popupFiles) )
+	if ( d->popupType != AlertHeadline && (d->popupType != AlertFile || !PsiOptions::instance()->getOption("options.ui.file-transfer.auto-popup").toBool()) )
 		setData(icon, contactText);
 	else if ( d->popupType == AlertHeadline ) {
 		QVBoxLayout *vbox = new QVBoxLayout(0);
@@ -387,7 +387,7 @@ void PsiPopup::setData(const Jid &j, const Resource &r, const UserListItem *u, c
 
 		QLabel *messageLabel = new QLabel(d->popup);
 		QFont font = messageLabel->font();
-		font.setPointSize(option.smallFontSize);
+		font.setPointSize(common_smallFontSize);
 		messageLabel->setFont(font);
 
 		messageLabel->setWordWrap(true);
@@ -414,7 +414,7 @@ void PsiPopup::show()
 		return;
 	}
 
-	if ( !d->id.isEmpty() /*&& option.ppNoDupes*/ ) {
+	if ( !d->id.isEmpty() /*&& LEGOPTS.ppNoDupes*/ ) {
 		foreach (PsiPopup *pp, *psiPopupList) {
 			if ( d->id == pp->id() && pp->popup() ) {
 				pp->popup()->restartHideTimer();
