@@ -1,5 +1,6 @@
 #include "spellhighlighter.h"
 #include "spellchecker.h"
+#include "common.h"
 
 SpellHighlighter::SpellHighlighter(QTextDocument* d) : QSyntaxHighlighter(d)
 {
@@ -10,7 +11,12 @@ void SpellHighlighter::highlightBlock(const QString& text)
 	// Underline 
 	QTextCharFormat tcf;
 	tcf.setUnderlineColor(QBrush(QColor(255,0,0)));
-	tcf.setUnderlineStyle(QTextCharFormat::SpellCheckUnderline);
+	if(qVersionInt() >= 0x040400) {
+		tcf.setUnderlineStyle(QTextCharFormat::DotLine);
+	}
+	else {
+		tcf.setUnderlineStyle(QTextCharFormat::SpellCheckUnderline);
+	}
 
 	// Match words (minimally)
 	QRegExp expression("\\b\\w+\\b");
